

#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <sys/timeb.h>
#include <math.h>


#define MAX_LENGTH 256
#define MAX_SIZE 5000

/* data structures ... (ha ha)
 */
typedef unsigned char Bool;
#define TRUE 1
#define FALSE 0



/* commands
 */
char gfilename[MAX_LENGTH];
int glowmargin;
int ghighmargin;
int gmaxBO;
int gmaxtuples;
int gonlycount;
int gonlymargin;
char ginstancesname[MAX_LENGTH];
Bool ginstancesname_given;
char ghardestname[MAX_LENGTH];
Bool ghardestname_given;
Bool gonlyrelevantBOs;
int gunslevel;
Bool gBOisolation;

int gnum_preds;
int gvar_pred[MAX_SIZE];
int gval_pred[MAX_SIZE];
int gBO_pred[MAX_SIZE];



/* command line
 */
void usage( void );
Bool process_command_line( int argc, char *argv[] );



void set_preprocess_false( void );
void output_PDDL( void );
void generate_PDDL_varsets( int BO, int n, int k );
void generate_PDDL_varvalues( int BO, int var );
void generate_BO_structures( void );
void output_one_PDDL_problem( int g );
void output_accumulated_PDDL_problem( int g );
void output_accumulated_single_PDDL_problem( int n );




int main( int argc, char *argv[] )

{

  FILE *fdata;

  char c;
  int i;

  gmaxBO = 1000;
  gmaxtuples = -1;
  glowmargin = 0;
  ghighmargin = 0;
  gonlycount = 1;
  gonlymargin = 1;
  ginstancesname_given = FALSE;
  ghardestname_given = FALSE;
  gonlyrelevantBOs = FALSE;
  gunslevel = 0; 
  gBOisolation = FALSE;
  if ( !process_command_line( argc, argv ) ) {
    usage();
    exit( 1 );
  }

  if ( (fdata = fopen( gfilename, "r")) == NULL ) {
    printf("\n\nCannot open file %s.\n\n", gfilename);
    exit( 1 );
  }

  gnum_preds = 0;
  while (TRUE ) {

    while ( (c=getc(fdata)) != 'r' && c != EOF );
    if ( c == EOF ) break;
    fscanf( fdata, "%d", &(gvar_pred[gnum_preds]));

    while ( (c=getc(fdata)) != 'l' && c != EOF );
    if ( c == EOF ) break;
    fscanf( fdata, "%d", &(gval_pred[gnum_preds]));

    while ( (c=getc(fdata)) != 'O' && c != EOF );
    if ( c == EOF ) break;
    fscanf( fdata, "%d", &(gBO_pred[gnum_preds]));

    gnum_preds++;

    while ( (c=getc(fdata)) != '\n');

  }
  fclose( fdata );

  printf("\ndata parsed:\n");
  for ( i = 0; i < gnum_preds; i++ ) {
    printf("p%d-Var%d-Val%d-BO%d\n",
	   i,
	   gvar_pred[i],
	   gval_pred[i],
	   gBO_pred[i]);
  } 

  output_PDDL();
  
  exit( 0 );

}











/*********************************************************
 *
 * here's the real work: output the PDDL files required
 * for this experiment
 *
 */




#define MAXBO 410
#define MAXBOVARS 20
#define MAXBOVARVALS 20
#define MAXT 10000

int num_BO;
int BO_var[MAXBO][MAXBOVARS];
int num_BO_var[MAXBO];
int BO_var_val[MAXBO][MAXBOVARS][MAXBOVARVALS];
int num_BO_var_val[MAXBO][MAXBOVARS];



/* this guy stores which single values are unreachable anyway
 * -- no point in generating those.
 *
 * this information was extracted "the hard way", from the FF data
 * generated for all single-var goals.
 */
gunsval[MAXBO][MAXBOVARS][MAXBOVARVALS];



int varon[MAXBOVARS];
int varset[MAXBOVARS];
int num_varset;
int valset[MAXBOVARS];

int count;

int valuetuples;

/* these guys store the "hardest" instance per BO, if given
 */
int hardestvarset[MAXBO][MAXBOVARS];
int hardestvalset[MAXBO][MAXBOVARS];
int num_hardestvarset[MAXBO];

/* this is a HACK! I haven't even verified 100%
 * whether this is correct -- although it seems it
 * must be.
 */
char goalconstantname[MAXBO][MAX_LENGTH];





void output_PDDL( void )

{

  int i, j, BO, digit;
  int max, min, maxBO;
  FILE *idata;
  char c;
  int digits[10], currentdigit;
  int k, ID;

  srandom( 13 );

  count = 0;

  generate_BO_structures();
  set_preprocess_false();




  if ( ginstancesname_given ) {
    /* simply read which instances are wanted, and skip all the rest.
     */
    if ( (idata = fopen( ginstancesname, "r")) == NULL ) {
      printf("\n\nCannot open file %s.\n\n", gfilename);
      exit( 1 );
    }
    while ( TRUE ) {
      
      /* parse the BO
       */
      while ( (c=getc(idata)) != 'O' && c != EOF );
      if ( c == EOF ) break;

      fscanf( idata, "%d", &BO);
      printf("parsing instance: p-BO%d", BO); fflush(stdout);
      
      /* parse the variables
       */
      while ( (c=getc(idata)) != 's' );
      printf("-Vars"); fflush(stdout);
      num_varset = 0;

      currentdigit = 0;
      while ( TRUE ) {

	/* read one var index; must make manually... brrr...
	 */
	while ( (c=getc(idata)) != '-' ) {
	  switch ( c ) {
	  case '0':
	    digit = 0;
	    break;
	  case '1':
	    digit = 1;
	    break;
	  case '2':
	    digit = 2;
	    break;
	  case '3':
	    digit = 3;
	    break;
	  case '4':
	    digit = 4;
	    break;
	  case '5':
	    digit = 5;
	    break;
	  case '6':
	    digit = 6;
	    break;
	  case '7':
	    digit = 7;
	    break;
	  case '8':
	    digit = 8;
	    break;
	  case '9':
	    digit = 9;
	    break;
	  default:
	    printf("\nunknown digit '%c'.\n\n", c);
	    exit(1);
	  }
	  digits[currentdigit++] = digit;
	  if ( currentdigit == 10 ) {
	    printf("\n10 digits?\n\n");
	    exit(1);
	  }
	}
	j = 1;
	varset[num_varset] = 0;
	for ( i = currentdigit-1; i >= 0; i-- ) {
	  varset[num_varset] += j * digits[i];
	  j *= 10;
	}
	printf("%d", varset[num_varset]); fflush(stdout);
	num_varset++;

	printf("-"); fflush(stdout);

	c=getc(idata);
	if ( c == 'V' ) {
	  printf("V"); fflush(stdout);
	  break;
	}

	/* this must be the first digit of the next var index.
	 */
	switch ( c ) {
	case '0':
	  digit = 0;
	  break;
	case '1':
	  digit = 1;
	  break;
	case '2':
	  digit = 2;
	  break;
	case '3':
	  digit = 3;
	  break;
	case '4':
	  digit = 4;
	  break;
	case '5':
	  digit = 5;
	  break;
	case '6':
	  digit = 6;
	  break;
	case '7':
	  digit = 7;
	  break;
	case '8':
	  digit = 8;
	  break;
	case '9':
	  digit = 9;
	  break;
	default:
	  printf("\nunknown digit '%c'.\n\n", c);
	  exit(1);
	}
	digits[0] = digit;
	currentdigit = 1;
      }

      /* parse the values
       */
      while ( (c=getc(idata)) != 's' );
      printf("als"); fflush(stdout);
      i = 0;
      while ( TRUE ) {
	fscanf( idata, "%d", &(valset[i]));
	printf("%d", valset[i]); fflush(stdout);
	i++;

	c=getc(idata);
	if ( c == '\n' ) {
	  printf("\n"); fflush(stdout);
	  break;
	}
	if ( c != '-' ) {
	  printf("\ndidn't find '-' where I expected it.\n\n");
	  exit(1);
	}
	printf("-"); fflush(stdout);
      }
      if ( i != num_varset ) {
	printf("\nnumber of vars != number of values?.\n\n");
	  exit(1);
      }


      /* now just directly output the PDDL.
       */
      output_one_PDDL_problem( BO );
    }

    fclose( idata );

    return;
  }








  if ( ghardestname_given ) {
    /* read the instances, generate the cross-BO tasks, and skip all the rest.
     */
    if ( (idata = fopen( ghardestname, "r")) == NULL ) {
      printf("\n\nCannot open file %s.\n\n", gfilename);
      exit( 1 );
    }
    ID = 0;
    while ( TRUE ) {
      
      /* parse the BO
       */
      while ( (c=getc(idata)) != 'O' && c != EOF );
      if ( c == EOF ) break;

      fscanf( idata, "%d", &BO);
      printf("parsing instance: p-BO%d", BO); fflush(stdout);
      if ( BO != ID ) {
	printf("\nBO != ID\n\n?");
	exit(1);
      }
      ID++;
      
      /* parse the variables
       */
      while ( (c=getc(idata)) != 's' );
      printf("-Vars"); fflush(stdout);
      num_varset = 0;

      currentdigit = 0;
      while ( TRUE ) {

	/* read one var index; must make manually... brrr...
	 */
	while ( (c=getc(idata)) != '-' ) {
	  switch ( c ) {
	  case '0':
	    digit = 0;
	    break;
	  case '1':
	    digit = 1;
	    break;
	  case '2':
	    digit = 2;
	    break;
	  case '3':
	    digit = 3;
	    break;
	  case '4':
	    digit = 4;
	    break;
	  case '5':
	    digit = 5;
	    break;
	  case '6':
	    digit = 6;
	    break;
	  case '7':
	    digit = 7;
	    break;
	  case '8':
	    digit = 8;
	    break;
	  case '9':
	    digit = 9;
	    break;
	  default:
	    printf("\nunknown digit '%c'.\n\n", c);
	    exit(1);
	  }
	  digits[currentdigit++] = digit;
	  if ( currentdigit == 10 ) {
	    printf("\n10 digits?\n\n");
	    exit(1);
	  }
	}
	j = 1;
	varset[num_varset] = 0;
	for ( i = currentdigit-1; i >= 0; i-- ) {
	  varset[num_varset] += j * digits[i];
	  j *= 10;
	}
	printf("%d", varset[num_varset]); fflush(stdout);
	num_varset++;

	printf("-"); fflush(stdout);

	c=getc(idata);
	if ( c == 'V' ) {
	  printf("V"); fflush(stdout);
	  break;
	}

	/* this must be the first digit of the next var index.
	 */
	switch ( c ) {
	case '0':
	  digit = 0;
	  break;
	case '1':
	  digit = 1;
	  break;
	case '2':
	  digit = 2;
	  break;
	case '3':
	  digit = 3;
	  break;
	case '4':
	  digit = 4;
	  break;
	case '5':
	  digit = 5;
	  break;
	case '6':
	  digit = 6;
	  break;
	case '7':
	  digit = 7;
	  break;
	case '8':
	  digit = 8;
	  break;
	case '9':
	  digit = 9;
	  break;
	default:
	  printf("\nunknown digit '%c'.\n\n", c);
	  exit(1);
	}
	digits[0] = digit;
	currentdigit = 1;
      }

      /* parse the values
       */
      while ( (c=getc(idata)) != 's' );
      printf("als"); fflush(stdout);
      i = 0;
      while ( TRUE ) {
	fscanf( idata, "%d", &(valset[i]));
	printf("%d", valset[i]); fflush(stdout);
	i++;

	c=getc(idata);
	if ( c == '\n' ) {
	  printf("\n"); fflush(stdout);
	  break;
	}
	if ( c != '-' ) {
	  printf("\ndidn't find '-' where I expected it.\n\n");
	  exit(1);
	}
	printf("-"); fflush(stdout);
      }
      if ( i != num_varset ) {
	printf("\nnumber of vars != number of values?.\n\n");
	  exit(1);
      }


      /* now store these guys.
       */
      for ( k = 0; k < num_varset; k++ ) {
	hardestvarset[BO][k] = varset[k];
	hardestvalset[BO][k] = valset[k];
      }
      num_hardestvarset[BO] = num_varset;
    }

    fclose( idata );


    /* now I have all these guys. what i want to do next is output
     * PDDL tasks accumulating the first k BO problems.
     */
    for ( k = 0; k < ID; k++ ) {
      output_accumulated_PDDL_problem( k );
      output_accumulated_single_PDDL_problem( k );

    }

    return;
  }



  maxBO = gmaxBO;
  if ( gmaxBO >= num_BO ) {
    maxBO = num_BO-1;
  }

  if ( gBOisolation ) {
    i = maxBO;
  } else {
    i = 0;
  }
  for ( ; i <= maxBO; i++ ) {
    if ( num_BO_var[i] == 0 ) {
      printf("\nBO with 0 vars?\n\n");
      exit(1);
    }
    
    for ( j = 0; j < num_BO_var[i]; j++ ) {
      varon[j] = 0;
    }

    max = glowmargin;
    if ( gonlymargin ) {
      if ( max <= num_BO_var[i] && max >= 1 ) {
	for ( j = max; j <= max; j++ ) {
	  /* generate all sets of vars with j elements
	   * whose indices start at the 2nd arg
	   */
	  generate_PDDL_varsets( i, j, 0 );
	}
      }
    } else {
      if ( max > num_BO_var[i] ) {
	max = num_BO_var[i];
      }
      for ( j = 1; j <= max; j++ ) {
	generate_PDDL_varsets( i, j, 0 );
      }
    }

    min = 1+num_BO_var[i]-ghighmargin;

    if ( gonlymargin ) {
      if ( min > num_BO_var[i] ) {
	continue;
      }
      if ( min < 1 ) {
	continue;
      }
      for ( j = min; j <= min; j++ ) {
	generate_PDDL_varsets( i, j, 0 );
      }
    } else {
      if ( min < 1 ) {
	min = 1;
      }
      for ( j = min; j <= num_BO_var[i]; j++ ) {
	generate_PDDL_varsets( i, j, 0 );
      }
    }
  }

/*   for ( i = 0; i < gnum_preds; i++ ) { */
/*     printf("\ngenerating problem: predicate %d\n", i); */
/*     output_one_PDDL_problem( i ); */
/*   }  */

  printf("\ncount: %d\n\n", count);

}



void generate_PDDL_varsets( int BO, int n, int k )

{

  int i, j, l;
  static int tuples[MAXT][MAXBOVARS];
  Bool prevonlycount;
  int prevmaxtuples;

  if ( n == 0 ) {
    num_varset = 0;
    for ( i = 0; i < num_BO_var[BO]; i++ ) {
      if ( varon[i] ) {
	varset[num_varset++] = i;
      }
    }

    /* generate all tuples of values for this var set
     * recursively: give index of var in varset to start with
     */
    if ( gmaxtuples == -1 ) {
      valuetuples = -1;
      generate_PDDL_varvalues( BO, 0 );
    } else {
      /* we have a limit. if the number of tuples here is below that limit,
       * then generate them all; else, take random samples (this is better
       * because it is not biased to just a few value combinations for the
       * vars at the start of the enumeration process)
       *
       * number tuples counted empirically because we don't take all
       * tuples -- just those that do not contain a preprocess-uns value.
       */
      
      /* here we act as if we only count, and there is no limit.
       * below we re-establish the actual settings.
       */
      prevonlycount = gonlycount;
      prevmaxtuples = gmaxtuples;
      gonlycount = TRUE;
      gmaxtuples = -1;
      valuetuples = 0;
      generate_PDDL_varvalues( BO, 0 );
      gonlycount = prevonlycount;
      gmaxtuples = prevmaxtuples;
      count -= valuetuples;
      
      if ( valuetuples <= gmaxtuples ) {
	/* set valuetuples to -1 to ensure that all tuples are
	 * enumerated
	 */
	valuetuples = -1;
	generate_PDDL_varvalues( BO, 0 );

      } else {

	for ( i = 0; i < gmaxtuples; i++ ) {
	  while ( 1 ) {
	    for ( j = 0; j < num_varset; j++ ) {
	      while ( 1 ) {
		valset[j] = random() % num_BO_var_val[BO][varset[j]];
		/* check whether this guy is uns
		 */
		if ( !gunsval[BO][varset[j]][valset[j]] ) {
		  break;
		}
	      } /* value selection */
	    } /* value tuple selection */
	    /* check whether we already had that one
	     */
	    for ( j = 0; j < i; j++ ) {
	      for ( l = 0; l < num_varset; l++ ) {
		if ( valset[l] != tuples[j][l] ) {
		  /* this guy is different!
		   */
		  break;
		}
	      }
	      if ( l == num_varset ) {
		/* this guy is the same!
		 */
		break;
	      }
	    }
	    if ( j == i ) break;
	  } /* end looping until new random tuple is found */
	  output_one_PDDL_problem( BO );
	  for ( l = 0; l < num_varset; l++ ) {
	    tuples[i][l] = valset[l];
	  }
	} /* endfor i over the rnd tuples to be generated */
      } /* endif #tuples here exceeds the limit */
    } /* endif there is a limit on #tuples */

    return;
  }

  for ( i = k; i < num_BO_var[BO]; i++ ) {
    varon[i] = 1;
    generate_PDDL_varsets( BO, n-1, i+1 );
    varon[i] = 0;
  }

}



/* here we assume that the relevant varset has been written.
 * now generate all value tuples. do so recursively, simply
 * one-by-one on vars.
 */
void generate_PDDL_varvalues( int BO, int var )

{

  int i;

  if ( var == num_varset ) {
    output_one_PDDL_problem( BO );

    if ( valuetuples != -1 ) {
      valuetuples++;
    }
    return;
  }
  
  for ( i = 0; i < num_BO_var_val[BO][varset[var]]; i++ ) {
    valset[var] = i;
    /* filter out unsat guys
     */
    if ( gunsval[BO][varset[var]][i] ) {
      continue;
    }
    generate_PDDL_varvalues( BO, var+1 );

    if ( gmaxtuples != -1 && valuetuples != -1 && 
	 valuetuples >= gmaxtuples ) {
      return;
    }
  }

}



void generate_BO_structures( void )

{

  int i, j, k;
  int BO, varID;

  num_BO = 0;
  for ( i = 0; i < MAXBO; i++ ) {
    num_BO_var[i] = 0;
    for ( j = 0; j < MAXBOVARS; j++ ) {
      num_BO_var_val[i][j] = 0; 
    }

  }


  BO = 0;
  for ( i = 0; i < gnum_preds; i++ ) {
    if ( gBO_pred[i] >= MAXBO ) {
      printf("\nMAXBO too small.\n\n");
      exit(1);
    }
    num_BO = gBO_pred[i]+1;
    BO = gBO_pred[i];
    
    for ( j = 0; j < num_BO_var[BO]; j++ ) {
      if ( BO_var[BO][j] == gvar_pred[i] ) break;
    }
    if ( j == num_BO_var[BO] ) {
      if ( num_BO_var[BO] >= MAXBOVARS ) {
	printf("\nMAXBOVARS too small.\n\n");
	exit(1);
      }      
      BO_var[BO][num_BO_var[BO]] = gvar_pred[i];
      num_BO_var[BO]++;
    }
    varID = j;
    
    for ( j = 0; j < num_BO_var_val[BO][varID]; j++ ) {
      if ( BO_var_val[BO][varID][j] == gval_pred[i] ) break;
    }
    if ( j == num_BO_var_val[BO][varID] ) {
      if ( num_BO_var_val[BO][varID] >= MAXBOVARVALS ) {
	printf("\nMAXBOVARVALS too small.\n\n");
	exit(1);
      }      
      BO_var_val[BO][varID][num_BO_var_val[BO][varID]] = gval_pred[i];
      num_BO_var_val[BO][varID]++;
    }
  } 
  


  printf("\n==============================================BO var vals parsed:");
  for ( i = 0; i < num_BO; i++ ) {
    printf("\n----------------------------------BO%d:", i);
    for ( j = 0; j < num_BO_var[i]; j++ ) {
      printf("\nVar%d: ", BO_var[i][j]);
      for ( k = 0; k < num_BO_var_val[i][j]; k++ ) {
	printf("%d ", BO_var_val[i][j][k]);
      }
    }
    printf("\n");
    
  }



}






























void usage( void )

{

  printf("\nusage:\n");

  printf("\nOPTIONS   DESCRIPTIONS\n\n");
  printf("-f <str>    name of file containing SAM predicates (required!)\n\n");
  printf("-i <str>    name of file containing names of instances to generate (if given, this over-rules all the below)\n");
  printf("-P          prune: generate only relevant BOs (not for use with -h)\n\n");
  printf("-h <str>    name of file containing names of hardest instance per BO (if given, this over-rules all the below)\n\n");
  printf("-U <num>    uns pruning level (default: %d)\n", gunslevel);
  printf("     0      all variable values\n");
  printf("     1      no var values preoved uns by pre-process\n");
  printf("     2      no var values proved unsat, including those where search is needed\n\n");
  printf("-B <num>    max BO number (default: %d)\n", gmaxBO);
  printf("-I          BO number in isolation; ie ONLY this BO\n\n");
  printf("-T <num>    max value tuples for any var combination (default: %d; max: %d)\n\n", 
	 gmaxtuples, MAXT);
  printf("-L <num>    lower margin (default: %d)\n", glowmargin);
  printf("-H <num>    higher margin (default: %d)\n", ghighmargin);
  printf("            sizes n of variable sets considered:\n");
  printf("            BO < B, and\n");
  printf("            1<=n<=l or 1+#vars-h<=n<=#vars\n");
  printf("            if -T != -1 then only T of each var combination are taken\n");
  printf("-M          inclusive margin for -LH (default: only margin itself)\n\n");
  printf("-G          generate PDDL (default: count problems only)\n\n");

}



Bool process_command_line( int argc, char *argv[] )

{

  char option;

  while ( --argc && ++argv ) {
    if ( *argv[0] != '-' || strlen(*argv) != 2 ) {
      return FALSE;
    }
    option = *++argv[0];
    switch ( option ) {
    case 'G':
      gonlycount = 0;
      break;
    case 'M':
      gonlymargin = 0;
      break;
    case 'P':
      gonlyrelevantBOs = TRUE;
      break;
    case 'I':
      gBOisolation = TRUE;
      break;
    default:
      if ( --argc && ++argv ) {
	switch ( option ) {
	case 'f':
	  strncpy( gfilename, *argv, MAX_LENGTH );
	  break;
	case 'i':
	  strncpy( ginstancesname, *argv, MAX_LENGTH );
	  ginstancesname_given = TRUE;
	  break;
	case 'h':
	  strncpy( ghardestname, *argv, MAX_LENGTH );
	  ghardestname_given = TRUE;
	  break;
	case 'U':
	  sscanf( *argv, "%d", &gunslevel );
	  break;	
	case 'L':
	  sscanf( *argv, "%d", &glowmargin );
	  break;	
	case 'H':
	  sscanf( *argv, "%d", &ghighmargin );
	  break;	
	case 'B':
	  sscanf( *argv, "%d", &gmaxBO );
	  break;	
	case 'T':
	  sscanf( *argv, "%d", &gmaxtuples );
	  break;	
	default:
	  printf( "\n\nunknown option: %c entered\n\n", option );
	  return FALSE;
	}
      } else {
	return FALSE;
      }
    }
  }

  if ( gmaxtuples > MAXT ) {
    return FALSE;
  }

  return TRUE;

}








































void output_one_PDDL_problem( int BO )

{

  FILE *fpddl;
  char filename[MAX_LENGTH], tmp[MAX_LENGTH];
  int i, j;

  count++;

  if ( gonlycount ) {
    return;
  }

  sprintf(filename, "p-BO%d-Vars", BO);
  for ( i = 0; i < num_varset; i++ ) {
    sprintf(tmp, "%d", varset[i]);
    if ( i < num_varset - 1 ) {
      strncat(tmp, "-", 1);
    }
    strncat(filename, tmp, strlen(tmp));
  }
  strncat(filename, "-Vals", 5);
  for ( i = 0; i < num_varset; i++ ) {
    sprintf(tmp, "%d", valset[i]);
    if ( i < num_varset - 1 ) {
      strncat(tmp, "-", 1);
    }
    strncat(filename, tmp, strlen(tmp));
  }

  printf("%7d %s\n", count, filename);

/*   if ( gonlycount ) { */
/*     return; */
/*   } */

  if ( (fpddl = fopen( filename, "w")) == NULL ) {
    printf("\n\nCannot open file %s.\n\n", filename);
    exit( 1 );
  }

  fprintf(fpddl, "; Modified from composer2pddl\n");
  fprintf(fpddl, ";              by Joerg Hoffmann, 2009\n\n");

  fprintf(fpddl, "(define (problem SAM)\n");
  fprintf(fpddl, "(:domain SAM)\n");
  fprintf(fpddl, "(:objects\n");
  if ( !gonlyrelevantBOs ) {
    for ( i = 0; i < 404; i++ ) {
      fprintf(fpddl, "BO%d - type-BO%d\n", i, i);
    }
  } else {
    fprintf(fpddl, "BO%d - type-BO%d\n", BO, BO);
  }
  fprintf(fpddl, ")\n");
  fprintf(fpddl, "(:init\n");
  fprintf(fpddl, "(foo)\n");
  fprintf(fpddl, ")\n");



  fprintf(fpddl, "(:goal\n");
  fprintf(fpddl, "(and\n");

  for ( i = 0; i < num_varset; i++ ) {
    for ( j = 0; j < gnum_preds; j++ ) {
      if ( gBO_pred[j] != BO ) continue;
      if ( gvar_pred[j] != BO_var[BO][varset[i]] ) continue;
      if ( gval_pred[j] != BO_var_val[BO][varset[i]][valset[i]] ) continue;
      break;
    }
    if ( j == gnum_preds ) {
      printf("\npredicate not found??\n\n");
      exit(1);
    }
    fprintf(fpddl, "(p%d-Var%d-Val%d-BO%d BO%d)\n",
	    j,
	    BO_var[BO][varset[i]],
	    BO_var_val[BO][varset[i]][valset[i]],
	    BO,
	    BO);
  }
  fprintf(fpddl, "(created BO%d)\n", BO);
  fprintf(fpddl, ")\n");
  fprintf(fpddl, ")\n");

  fprintf(fpddl, ")\n");
  
  fclose( fpddl );

}




void output_accumulated_PDDL_problem( int n )

{

  FILE *fpddl;
  char filename[MAX_LENGTH];
  int i, j, BO;


  count++;

  if ( gonlycount ) {
    return;
  }

  sprintf(filename, "p-acc-%d", n);
  printf("%7d %s\n", count, filename);


  if ( (fpddl = fopen( filename, "w")) == NULL ) {
    printf("\n\nCannot open file %s.\n\n", filename);
    exit( 1 );
  }

  fprintf(fpddl, "; Modified from composer2pddl\n");
  fprintf(fpddl, ";              by Joerg Hoffmann, 2009\n\n");

  fprintf(fpddl, "(define (problem SAM)\n");
  fprintf(fpddl, "(:domain SAM)\n");
  fprintf(fpddl, "(:objects\n");
  for ( i = 0; i < 404; i++ ) {
    fprintf(fpddl, "BO%d - type-BO%d\n", i, i);
  }
  fprintf(fpddl, ")\n");
  fprintf(fpddl, "(:init\n");
  fprintf(fpddl, "(foo)\n");
  fprintf(fpddl, ")\n");

  fprintf(fpddl, "(:goal\n");
  fprintf(fpddl, "(and\n");
  for ( BO = 0; BO <= n; BO++ ) {
    for ( i = 0; i < num_hardestvarset[BO]; i++ ) {
      for ( j = 0; j < gnum_preds; j++ ) {
	if ( gBO_pred[j] != BO ) continue;
	if ( gvar_pred[j] != BO_var[BO][hardestvarset[BO][i]] ) continue;
	if ( gval_pred[j] != BO_var_val[BO][hardestvarset[BO][i]][hardestvalset[BO][i]] ) continue;
	break;
      }
      if ( j == gnum_preds ) {
	printf("\npredicate not found??\n\n");
	exit(1);
      }
      fprintf(fpddl, "(p%d-Var%d-Val%d-BO%d BO%d)\n",
	      j,
	      BO_var[BO][hardestvarset[BO][i]],
	      BO_var_val[BO][hardestvarset[BO][i]][hardestvalset[BO][i]],
	      BO,
	      BO);
    }
    fprintf(fpddl, "(created BO%d)\n", BO);
  }
  fprintf(fpddl, ")\n");
  fprintf(fpddl, ")\n")
;
  fprintf(fpddl, ")\n");
  
  fclose( fpddl );


}




void output_accumulated_single_PDDL_problem( int n )

{

  FILE *fpddl;
  char filename[MAX_LENGTH];
  int i, j, BO;


  count++;

  if ( gonlycount ) {
    return;
  }

  sprintf(filename, "p-%d", n);
  printf("%7d %s\n", count, filename);


  if ( (fpddl = fopen( filename, "w")) == NULL ) {
    printf("\n\nCannot open file %s.\n\n", filename);
    exit( 1 );
  }

  fprintf(fpddl, "; Modified from composer2pddl\n");
  fprintf(fpddl, ";              by Joerg Hoffmann, 2009\n\n");

  fprintf(fpddl, "(define (problem SAM)\n");
  fprintf(fpddl, "(:domain SAM)\n");
  fprintf(fpddl, "(:objects\n");
  for ( i = 0; i < 404; i++ ) {
    fprintf(fpddl, "BO%d - type-BO%d\n", i, i);
  }
  fprintf(fpddl, ")\n");
  fprintf(fpddl, "(:init\n");
  fprintf(fpddl, "(foo)\n");
  fprintf(fpddl, ")\n");

  fprintf(fpddl, "(:goal\n");
  fprintf(fpddl, "(and\n");
  for ( BO = n; BO <= n; BO++ ) {
    for ( i = 0; i < num_hardestvarset[BO]; i++ ) {
      for ( j = 0; j < gnum_preds; j++ ) {
	if ( gBO_pred[j] != BO ) continue;
	if ( gvar_pred[j] != BO_var[BO][hardestvarset[BO][i]] ) continue;
	if ( gval_pred[j] != BO_var_val[BO][hardestvarset[BO][i]][hardestvalset[BO][i]] ) continue;
	break;
      }
      if ( j == gnum_preds ) {
	printf("\npredicate not found??\n\n");
	exit(1);
      }
      fprintf(fpddl, "(p%d-Var%d-Val%d-BO%d BO%d)\n",
	      j,
	      BO_var[BO][hardestvarset[BO][i]],
	      BO_var_val[BO][hardestvarset[BO][i]][hardestvalset[BO][i]],
	      BO,
	      BO);
    }
    fprintf(fpddl, "(created BO%d)\n", BO);
  }
  fprintf(fpddl, ")\n");
  fprintf(fpddl, ")\n")
;
  fprintf(fpddl, ")\n");
  
  fclose( fpddl );


}




















void set_preprocess_false( void )

{

  int i, j, k;

  
  for ( i = 0; i < num_BO; i++ ) {
    for ( j = 0; j < num_BO_var[i]; j++ ) {
      for ( k = 0; k < num_BO_var_val[i][j]; k++ ) {
	gunsval[i][j][k] = FALSE;
      }
    }
  }


  if ( gunslevel == 1 ) {
    gunsval[0][5][0] = TRUE;
    gunsval[0][5][1] = TRUE;
    gunsval[0][5][2] = TRUE;
    gunsval[0][5][3] = TRUE;
    gunsval[0][5][4] = TRUE;
    gunsval[0][5][5] = TRUE;
    gunsval[100][1][0] = TRUE;
    gunsval[100][2][0] = TRUE;
    gunsval[101][0][2] = TRUE;
    gunsval[101][1][0] = TRUE;
    gunsval[101][1][10] = TRUE;
    gunsval[101][1][2] = TRUE;
    gunsval[101][1][3] = TRUE;
    gunsval[101][1][4] = TRUE;
    gunsval[101][1][5] = TRUE;
    gunsval[101][1][6] = TRUE;
    gunsval[101][1][7] = TRUE;
    gunsval[101][1][8] = TRUE;
    gunsval[101][1][9] = TRUE;
    gunsval[101][2][0] = TRUE;
    gunsval[101][2][1] = TRUE;
    gunsval[101][3][0] = TRUE;
    gunsval[101][3][1] = TRUE;
    gunsval[101][3][10] = TRUE;
    gunsval[101][3][11] = TRUE;
    gunsval[101][3][2] = TRUE;
    gunsval[101][3][3] = TRUE;
    gunsval[101][3][4] = TRUE;
    gunsval[101][3][5] = TRUE;
    gunsval[101][3][7] = TRUE;
    gunsval[101][3][8] = TRUE;
    gunsval[101][3][9] = TRUE;
    gunsval[102][1][1] = TRUE;
    gunsval[102][1][2] = TRUE;
    gunsval[102][1][3] = TRUE;
    gunsval[102][1][4] = TRUE;
    gunsval[102][1][5] = TRUE;
    gunsval[102][1][6] = TRUE;
    gunsval[104][6][1] = TRUE;
    gunsval[104][6][2] = TRUE;
    gunsval[106][1][0] = TRUE;
    gunsval[106][1][1] = TRUE;
    gunsval[106][1][3] = TRUE;
    gunsval[106][4][1] = TRUE;
    gunsval[106][4][10] = TRUE;
    gunsval[106][4][2] = TRUE;
    gunsval[106][4][3] = TRUE;
    gunsval[106][4][4] = TRUE;
    gunsval[106][4][5] = TRUE;
    gunsval[106][4][6] = TRUE;
    gunsval[106][4][7] = TRUE;
    gunsval[106][4][8] = TRUE;
    gunsval[106][4][9] = TRUE;
    gunsval[106][6][1] = TRUE;
    gunsval[106][6][2] = TRUE;
    gunsval[107][3][0] = TRUE;
    gunsval[107][3][1] = TRUE;
    gunsval[107][3][2] = TRUE;
    gunsval[107][6][0] = TRUE;
    gunsval[107][6][1] = TRUE;
    gunsval[107][6][3] = TRUE;
    gunsval[107][6][4] = TRUE;
    gunsval[107][6][5] = TRUE;
    gunsval[107][7][0] = TRUE;
    gunsval[107][7][2] = TRUE;
    gunsval[108][0][0] = TRUE;
    gunsval[108][0][1] = TRUE;
    gunsval[108][0][3] = TRUE;
    gunsval[108][0][4] = TRUE;
    gunsval[108][0][5] = TRUE;
    gunsval[108][0][6] = TRUE;
    gunsval[108][1][0] = TRUE;
    gunsval[108][1][1] = TRUE;
    gunsval[108][1][2] = TRUE;
    gunsval[108][1][4] = TRUE;
    gunsval[108][1][5] = TRUE;
    gunsval[108][1][6] = TRUE;
    gunsval[109][1][1] = TRUE;
    gunsval[109][1][2] = TRUE;
    gunsval[109][1][3] = TRUE;
    gunsval[109][1][4] = TRUE;
    gunsval[109][1][5] = TRUE;
    gunsval[109][1][6] = TRUE;
    gunsval[109][1][7] = TRUE;
    gunsval[109][1][8] = TRUE;
    gunsval[109][2][0] = TRUE;
    gunsval[109][2][1] = TRUE;
    gunsval[109][2][2] = TRUE;
    gunsval[109][2][3] = TRUE;
    gunsval[109][2][5] = TRUE;
    gunsval[109][2][6] = TRUE;
    gunsval[109][2][7] = TRUE;
    gunsval[109][2][8] = TRUE;
    gunsval[10][1][0] = TRUE;
    gunsval[10][2][0] = TRUE;
    gunsval[10][2][1] = TRUE;
    gunsval[113][7][0] = TRUE;
    gunsval[113][7][1] = TRUE;
    gunsval[113][7][2] = TRUE;
    gunsval[113][7][3] = TRUE;
    gunsval[113][7][5] = TRUE;
    gunsval[113][7][6] = TRUE;
    gunsval[115][1][0] = TRUE;
    gunsval[115][2][1] = TRUE;
    gunsval[115][2][2] = TRUE;
    gunsval[115][4][0] = TRUE;
    gunsval[117][2][0] = TRUE;
    gunsval[117][2][1] = TRUE;
    gunsval[117][2][2] = TRUE;
    gunsval[117][2][3] = TRUE;
    gunsval[117][2][4] = TRUE;
    gunsval[117][2][5] = TRUE;
    gunsval[117][2][6] = TRUE;
    gunsval[117][2][7] = TRUE;
    gunsval[117][2][8] = TRUE;
    gunsval[117][2][9] = TRUE;
    gunsval[117][3][0] = TRUE;
    gunsval[117][3][2] = TRUE;
    gunsval[117][4][1] = TRUE;
    gunsval[117][4][2] = TRUE;
    gunsval[117][5][0] = TRUE;
    gunsval[117][5][2] = TRUE;
    gunsval[118][0][0] = TRUE;
    gunsval[118][0][1] = TRUE;
    gunsval[118][0][3] = TRUE;
    gunsval[118][1][0] = TRUE;
    gunsval[118][1][2] = TRUE;
    gunsval[118][1][3] = TRUE;
    gunsval[118][1][4] = TRUE;
    gunsval[118][1][5] = TRUE;
    gunsval[118][1][6] = TRUE;
    gunsval[118][1][7] = TRUE;
    gunsval[118][1][8] = TRUE;
    gunsval[119][8][0] = TRUE;
    gunsval[119][8][1] = TRUE;
    gunsval[119][8][2] = TRUE;
    gunsval[119][8][4] = TRUE;
    gunsval[119][8][5] = TRUE;
    gunsval[119][8][6] = TRUE;
    gunsval[119][8][7] = TRUE;
    gunsval[119][8][8] = TRUE;
    gunsval[119][8][9] = TRUE;
    gunsval[120][10][0] = TRUE;
    gunsval[120][10][1] = TRUE;
    gunsval[120][10][2] = TRUE;
    gunsval[120][10][3] = TRUE;
    gunsval[120][10][4] = TRUE;
    gunsval[120][10][6] = TRUE;
    gunsval[120][10][7] = TRUE;
    gunsval[120][10][8] = TRUE;
    gunsval[120][10][9] = TRUE;
    gunsval[120][1][0] = TRUE;
    gunsval[120][4][0] = TRUE;
    gunsval[120][4][1] = TRUE;
    gunsval[120][4][2] = TRUE;
    gunsval[120][5][0] = TRUE;
    gunsval[120][5][1] = TRUE;
    gunsval[120][5][2] = TRUE;
    gunsval[120][6][0] = TRUE;
    gunsval[120][6][2] = TRUE;
    gunsval[120][6][3] = TRUE;
    gunsval[120][8][0] = TRUE;
    gunsval[120][8][1] = TRUE;
    gunsval[120][8][2] = TRUE;
    gunsval[120][8][3] = TRUE;
    gunsval[120][8][5] = TRUE;
    gunsval[120][8][6] = TRUE;
    gunsval[120][8][7] = TRUE;
    gunsval[120][8][8] = TRUE;
    gunsval[120][9][0] = TRUE;
    gunsval[120][9][2] = TRUE;
    gunsval[120][9][3] = TRUE;
    gunsval[121][5][0] = TRUE;
    gunsval[121][5][1] = TRUE;
    gunsval[121][5][2] = TRUE;
    gunsval[121][5][3] = TRUE;
    gunsval[121][5][4] = TRUE;
    gunsval[121][5][6] = TRUE;
    gunsval[121][5][7] = TRUE;
    gunsval[121][5][8] = TRUE;
    gunsval[121][5][9] = TRUE;
    gunsval[121][6][0] = TRUE;
    gunsval[121][6][1] = TRUE;
    gunsval[125][5][0] = TRUE;
    gunsval[125][5][1] = TRUE;
    gunsval[125][5][3] = TRUE;
    gunsval[125][5][4] = TRUE;
    gunsval[126][2][0] = TRUE;
    gunsval[126][2][1] = TRUE;
    gunsval[126][2][3] = TRUE;
    gunsval[126][2][4] = TRUE;
    gunsval[127][1][2] = TRUE;
    gunsval[127][1][3] = TRUE;
    gunsval[127][1][4] = TRUE;
    gunsval[128][0][0] = TRUE;
    gunsval[128][0][1] = TRUE;
    gunsval[128][0][2] = TRUE;
    gunsval[129][3][0] = TRUE;
    gunsval[129][3][1] = TRUE;
    gunsval[129][3][2] = TRUE;
    gunsval[12][2][0] = TRUE;
    gunsval[12][2][2] = TRUE;
    gunsval[12][3][1] = TRUE;
    gunsval[12][3][2] = TRUE;
    gunsval[12][3][3] = TRUE;
    gunsval[136][2][0] = TRUE;
    gunsval[136][2][1] = TRUE;
    gunsval[137][6][0] = TRUE;
    gunsval[138][0][0] = TRUE;
    gunsval[138][0][1] = TRUE;
    gunsval[138][3][0] = TRUE;
    gunsval[138][3][1] = TRUE;
    gunsval[138][3][2] = TRUE;
    gunsval[138][3][3] = TRUE;
    gunsval[138][3][5] = TRUE;
    gunsval[138][3][6] = TRUE;
    gunsval[138][6][0] = TRUE;
    gunsval[138][6][1] = TRUE;
    gunsval[139][2][0] = TRUE;
    gunsval[139][2][1] = TRUE;
    gunsval[139][5][0] = TRUE;
    gunsval[139][5][2] = TRUE;
    gunsval[139][6][0] = TRUE;
    gunsval[139][6][1] = TRUE;
    gunsval[139][7][1] = TRUE;
    gunsval[139][7][2] = TRUE;
    gunsval[139][7][3] = TRUE;
    gunsval[13][0][0] = TRUE;
    gunsval[13][1][0] = TRUE;
    gunsval[13][1][2] = TRUE;
    gunsval[141][10][0] = TRUE;
    gunsval[141][10][2] = TRUE;
    gunsval[141][10][3] = TRUE;
    gunsval[141][11][0] = TRUE;
    gunsval[141][11][2] = TRUE;
    gunsval[141][11][3] = TRUE;
    gunsval[141][12][0] = TRUE;
    gunsval[141][12][1] = TRUE;
    gunsval[141][13][0] = TRUE;
    gunsval[141][13][2] = TRUE;
    gunsval[141][13][3] = TRUE;
    gunsval[141][14][1] = TRUE;
    gunsval[141][14][2] = TRUE;
    gunsval[141][14][3] = TRUE;
    gunsval[141][2][0] = TRUE;
    gunsval[141][2][1] = TRUE;
    gunsval[141][8][1] = TRUE;
    gunsval[141][8][2] = TRUE;
    gunsval[141][9][0] = TRUE;
    gunsval[141][9][2] = TRUE;
    gunsval[142][4][0] = TRUE;
    gunsval[142][4][2] = TRUE;
    gunsval[142][4][3] = TRUE;
    gunsval[142][9][0] = TRUE;
    gunsval[142][9][2] = TRUE;
    gunsval[143][3][0] = TRUE;
    gunsval[143][3][1] = TRUE;
    gunsval[144][10][0] = TRUE;
    gunsval[144][10][2] = TRUE;
    gunsval[144][7][0] = TRUE;
    gunsval[144][7][2] = TRUE;
    gunsval[144][7][3] = TRUE;
    gunsval[145][1][0] = TRUE;
    gunsval[145][1][1] = TRUE;
    gunsval[145][5][0] = TRUE;
    gunsval[145][5][1] = TRUE;
    gunsval[145][6][0] = TRUE;
    gunsval[145][6][2] = TRUE;
    gunsval[145][7][0] = TRUE;
    gunsval[145][7][1] = TRUE;
    gunsval[145][8][0] = TRUE;
    gunsval[145][8][1] = TRUE;
    gunsval[146][10][0] = TRUE;
    gunsval[146][10][1] = TRUE;
    gunsval[146][11][1] = TRUE;
    gunsval[146][11][2] = TRUE;
    gunsval[146][12][1] = TRUE;
    gunsval[146][12][2] = TRUE;
    gunsval[146][12][3] = TRUE;
    gunsval[146][1][0] = TRUE;
    gunsval[146][1][1] = TRUE;
    gunsval[146][1][2] = TRUE;
    gunsval[146][5][0] = TRUE;
    gunsval[146][5][1] = TRUE;
    gunsval[146][7][0] = TRUE;
    gunsval[146][7][2] = TRUE;
    gunsval[146][8][0] = TRUE;
    gunsval[146][8][2] = TRUE;
    gunsval[146][8][3] = TRUE;
    gunsval[146][9][1] = TRUE;
    gunsval[146][9][2] = TRUE;
    gunsval[146][9][3] = TRUE;
    gunsval[147][11][0] = TRUE;
    gunsval[147][11][2] = TRUE;
    gunsval[147][6][0] = TRUE;
    gunsval[147][6][2] = TRUE;
    gunsval[147][6][3] = TRUE;
    gunsval[148][10][0] = TRUE;
    gunsval[148][10][2] = TRUE;
    gunsval[148][10][3] = TRUE;
    gunsval[148][12][0] = TRUE;
    gunsval[148][12][2] = TRUE;
    gunsval[149][3][0] = TRUE;
    gunsval[149][3][2] = TRUE;
    gunsval[149][4][0] = TRUE;
    gunsval[149][4][1] = TRUE;
    gunsval[149][5][0] = TRUE;
    gunsval[149][5][1] = TRUE;
    gunsval[149][6][0] = TRUE;
    gunsval[149][6][1] = TRUE;
    gunsval[149][6][2] = TRUE;
    gunsval[14][1][0] = TRUE;
    gunsval[14][2][0] = TRUE;
    gunsval[14][2][1] = TRUE;
    gunsval[157][3][1] = TRUE;
    gunsval[157][3][2] = TRUE;
    gunsval[157][4][0] = TRUE;
    gunsval[157][4][1] = TRUE;
    gunsval[157][4][3] = TRUE;
    gunsval[157][4][4] = TRUE;
    gunsval[157][5][0] = TRUE;
    gunsval[157][5][2] = TRUE;
    gunsval[157][6][1] = TRUE;
    gunsval[157][6][2] = TRUE;
    gunsval[159][3][1] = TRUE;
    gunsval[159][3][2] = TRUE;
    gunsval[159][4][0] = TRUE;
    gunsval[159][4][1] = TRUE;
    gunsval[159][5][0] = TRUE;
    gunsval[15][0][0] = TRUE;
    gunsval[15][1][0] = TRUE;
    gunsval[15][1][1] = TRUE;
    gunsval[167][0][0] = TRUE;
    gunsval[167][0][2] = TRUE;
    gunsval[168][1][0] = TRUE;
    gunsval[168][1][1] = TRUE;
    gunsval[168][3][0] = TRUE;
    gunsval[168][3][1] = TRUE;
    gunsval[168][3][3] = TRUE;
    gunsval[176][1][1] = TRUE;
    gunsval[176][2][0] = TRUE;
    gunsval[176][2][1] = TRUE;
    gunsval[179][1][0] = TRUE;
    gunsval[17][1][0] = TRUE;
    gunsval[17][1][1] = TRUE;
    gunsval[17][1][3] = TRUE;
    gunsval[17][2][0] = TRUE;
    gunsval[17][2][2] = TRUE;
    gunsval[181][2][0] = TRUE;
    gunsval[181][2][1] = TRUE;
    gunsval[181][2][3] = TRUE;
    gunsval[182][0][0] = TRUE;
    gunsval[182][0][1] = TRUE;
    gunsval[182][0][2] = TRUE;
    gunsval[182][0][3] = TRUE;
    gunsval[182][1][0] = TRUE;
    gunsval[182][1][1] = TRUE;
    gunsval[182][2][0] = TRUE;
    gunsval[182][2][1] = TRUE;
    gunsval[182][2][2] = TRUE;
    gunsval[182][2][3] = TRUE;
    gunsval[182][2][4] = TRUE;
    gunsval[182][2][5] = TRUE;
    gunsval[183][0][0] = TRUE;
    gunsval[183][0][2] = TRUE;
    gunsval[183][1][0] = TRUE;
    gunsval[183][1][1] = TRUE;
    gunsval[183][2][0] = TRUE;
    gunsval[183][2][1] = TRUE;
    gunsval[183][2][2] = TRUE;
    gunsval[183][2][4] = TRUE;
    gunsval[184][2][0] = TRUE;
    gunsval[184][2][1] = TRUE;
    gunsval[184][2][2] = TRUE;
    gunsval[184][2][3] = TRUE;
    gunsval[184][2][4] = TRUE;
    gunsval[184][2][6] = TRUE;
    gunsval[187][2][0] = TRUE;
    gunsval[187][4][0] = TRUE;
    gunsval[187][4][1] = TRUE;
    gunsval[187][4][2] = TRUE;
    gunsval[187][4][4] = TRUE;
    gunsval[188][0][0] = TRUE;
    gunsval[188][1][0] = TRUE;
    gunsval[188][1][1] = TRUE;
    gunsval[188][1][3] = TRUE;
    gunsval[188][3][0] = TRUE;
    gunsval[188][3][1] = TRUE;
    gunsval[188][3][2] = TRUE;
    gunsval[188][3][4] = TRUE;
    gunsval[189][0][0] = TRUE;
    gunsval[189][1][0] = TRUE;
    gunsval[189][2][0] = TRUE;
    gunsval[189][2][1] = TRUE;
    gunsval[189][2][2] = TRUE;
    gunsval[189][4][0] = TRUE;
    gunsval[189][4][1] = TRUE;
    gunsval[189][4][2] = TRUE;
    gunsval[189][4][3] = TRUE;
    gunsval[189][4][5] = TRUE;
    gunsval[18][1][0] = TRUE;
    gunsval[18][1][1] = TRUE;
    gunsval[18][1][2] = TRUE;
    gunsval[18][3][0] = TRUE;
    gunsval[18][3][1] = TRUE;
    gunsval[190][1][0] = TRUE;
    gunsval[190][3][0] = TRUE;
    gunsval[190][3][1] = TRUE;
    gunsval[190][3][3] = TRUE;
    gunsval[191][1][0] = TRUE;
    gunsval[191][1][1] = TRUE;
    gunsval[192][0][2] = TRUE;
    gunsval[192][1][0] = TRUE;
    gunsval[192][2][0] = TRUE;
    gunsval[192][3][0] = TRUE;
    gunsval[192][4][0] = TRUE;
    gunsval[192][4][2] = TRUE;
    gunsval[192][4][3] = TRUE;
    gunsval[192][4][4] = TRUE;
    gunsval[193][0][1] = TRUE;
    gunsval[193][0][2] = TRUE;
    gunsval[193][2][1] = TRUE;
    gunsval[193][2][2] = TRUE;
    gunsval[194][1][1] = TRUE;
    gunsval[198][3][1] = TRUE;
    gunsval[198][3][2] = TRUE;
    gunsval[198][3][3] = TRUE;
    gunsval[19][1][0] = TRUE;
    gunsval[19][1][1] = TRUE;
    gunsval[19][1][3] = TRUE;
    gunsval[19][2][0] = TRUE;
    gunsval[19][2][1] = TRUE;
    gunsval[1][0][0] = TRUE;
    gunsval[1][0][1] = TRUE;
    gunsval[1][6][0] = TRUE;
    gunsval[1][6][1] = TRUE;
    gunsval[1][6][2] = TRUE;
    gunsval[1][6][3] = TRUE;
    gunsval[1][6][4] = TRUE;
    gunsval[1][6][5] = TRUE;
    gunsval[200][2][0] = TRUE;
    gunsval[200][2][1] = TRUE;
    gunsval[205][0][0] = TRUE;
    gunsval[205][0][2] = TRUE;
    gunsval[205][1][1] = TRUE;
    gunsval[207][2][0] = TRUE;
    gunsval[207][2][1] = TRUE;
    gunsval[207][2][3] = TRUE;
    gunsval[207][2][4] = TRUE;
    gunsval[208][0][1] = TRUE;
    gunsval[208][0][2] = TRUE;
    gunsval[209][4][0] = TRUE;
    gunsval[209][4][1] = TRUE;
    gunsval[209][4][2] = TRUE;
    gunsval[209][4][4] = TRUE;
    gunsval[209][6][0] = TRUE;
    gunsval[209][6][1] = TRUE;
    gunsval[210][5][0] = TRUE;
    gunsval[210][5][1] = TRUE;
    gunsval[210][5][3] = TRUE;
    gunsval[210][5][4] = TRUE;
    gunsval[210][6][0] = TRUE;
    gunsval[210][6][1] = TRUE;
    gunsval[219][5][0] = TRUE;
    gunsval[219][5][1] = TRUE;
    gunsval[21][2][0] = TRUE;
    gunsval[220][4][0] = TRUE;
    gunsval[220][4][1] = TRUE;
    gunsval[220][4][2] = TRUE;
    gunsval[220][4][3] = TRUE;
    gunsval[220][4][4] = TRUE;
    gunsval[220][4][5] = TRUE;
    gunsval[220][4][7] = TRUE;
    gunsval[220][4][8] = TRUE;
    gunsval[221][4][0] = TRUE;
    gunsval[221][4][1] = TRUE;
    gunsval[221][4][2] = TRUE;
    gunsval[221][4][3] = TRUE;
    gunsval[221][4][4] = TRUE;
    gunsval[221][4][5] = TRUE;
    gunsval[221][4][7] = TRUE;
    gunsval[221][4][8] = TRUE;
    gunsval[222][2][0] = TRUE;
    gunsval[223][4][0] = TRUE;
    gunsval[223][4][1] = TRUE;
    gunsval[223][4][2] = TRUE;
    gunsval[223][4][3] = TRUE;
    gunsval[223][4][4] = TRUE;
    gunsval[223][4][6] = TRUE;
    gunsval[224][1][0] = TRUE;
    gunsval[224][1][1] = TRUE;
    gunsval[238][0][0] = TRUE;
    gunsval[238][0][2] = TRUE;
    gunsval[238][0][3] = TRUE;
    gunsval[238][0][4] = TRUE;
    gunsval[238][1][0] = TRUE;
    gunsval[238][1][1] = TRUE;
    gunsval[238][1][3] = TRUE;
    gunsval[238][1][4] = TRUE;
    gunsval[240][0][1] = TRUE;
    gunsval[240][0][2] = TRUE;
    gunsval[240][0][3] = TRUE;
    gunsval[240][0][4] = TRUE;
    gunsval[246][4][0] = TRUE;
    gunsval[246][6][0] = TRUE;
    gunsval[247][3][0] = TRUE;
    gunsval[248][0][0] = TRUE;
    gunsval[248][2][0] = TRUE;
    gunsval[248][2][2] = TRUE;
    gunsval[248][3][0] = TRUE;
    gunsval[248][4][0] = TRUE;
    gunsval[248][4][2] = TRUE;
    gunsval[248][5][0] = TRUE;
    gunsval[249][2][0] = TRUE;
    gunsval[249][2][2] = TRUE;
    gunsval[249][4][0] = TRUE;
    gunsval[249][5][0] = TRUE;
    gunsval[249][5][1] = TRUE;
    gunsval[249][5][2] = TRUE;
    gunsval[249][5][4] = TRUE;
    gunsval[249][5][5] = TRUE;
    gunsval[249][6][1] = TRUE;
    gunsval[250][0][0] = TRUE;
    gunsval[250][2][0] = TRUE;
    gunsval[250][2][1] = TRUE;
    gunsval[250][3][0] = TRUE;
    gunsval[250][4][1] = TRUE;
    gunsval[250][5][0] = TRUE;
    gunsval[250][6][0] = TRUE;
    gunsval[250][6][2] = TRUE;
    gunsval[251][0][0] = TRUE;
    gunsval[251][1][0] = TRUE;
    gunsval[251][1][2] = TRUE;
    gunsval[251][2][1] = TRUE;
    gunsval[251][2][2] = TRUE;
    gunsval[251][4][0] = TRUE;
    gunsval[251][5][0] = TRUE;
    gunsval[251][6][0] = TRUE;
    gunsval[251][6][2] = TRUE;
    gunsval[251][7][1] = TRUE;
    gunsval[251][7][3] = TRUE;
    gunsval[252][2][0] = TRUE;
    gunsval[252][4][0] = TRUE;
    gunsval[252][5][0] = TRUE;
    gunsval[254][0][0] = TRUE;
    gunsval[254][5][0] = TRUE;
    gunsval[256][0][1] = TRUE;
    gunsval[256][0][2] = TRUE;
    gunsval[256][1][0] = TRUE;
    gunsval[256][1][3] = TRUE;
    gunsval[256][2][0] = TRUE;
    gunsval[256][2][1] = TRUE;
    gunsval[256][3][0] = TRUE;
    gunsval[256][3][1] = TRUE;
    gunsval[256][5][0] = TRUE;
    gunsval[259][1][1] = TRUE;
    gunsval[259][1][2] = TRUE;
    gunsval[259][2][0] = TRUE;
    gunsval[259][3][0] = TRUE;
    gunsval[259][3][1] = TRUE;
    gunsval[259][3][3] = TRUE;
    gunsval[260][3][0] = TRUE;
    gunsval[261][0][0] = TRUE;
    gunsval[261][1][0] = TRUE;
    gunsval[261][1][1] = TRUE;
    gunsval[261][2][0] = TRUE;
    gunsval[261][2][1] = TRUE;
    gunsval[261][2][3] = TRUE;
    gunsval[261][2][4] = TRUE;
    gunsval[261][3][1] = TRUE;
    gunsval[261][3][2] = TRUE;
    gunsval[262][3][0] = TRUE;
    gunsval[262][3][1] = TRUE;
    gunsval[264][0][0] = TRUE;
    gunsval[264][0][1] = TRUE;
    gunsval[264][1][0] = TRUE;
    gunsval[264][1][4] = TRUE;
    gunsval[264][2][0] = TRUE;
    gunsval[264][2][1] = TRUE;
    gunsval[265][0][0] = TRUE;
    gunsval[265][0][1] = TRUE;
    gunsval[265][0][2] = TRUE;
    gunsval[265][1][0] = TRUE;
    gunsval[265][1][1] = TRUE;
    gunsval[265][1][3] = TRUE;
    gunsval[265][1][4] = TRUE;
    gunsval[265][2][0] = TRUE;
    gunsval[267][0][0] = TRUE;
    gunsval[267][0][1] = TRUE;
    gunsval[267][1][0] = TRUE;
    gunsval[267][1][4] = TRUE;
    gunsval[267][2][0] = TRUE;
    gunsval[267][2][1] = TRUE;
    gunsval[268][0][1] = TRUE;
    gunsval[268][0][2] = TRUE;
    gunsval[268][1][0] = TRUE;
    gunsval[268][2][0] = TRUE;
    gunsval[268][2][1] = TRUE;
    gunsval[269][0][0] = TRUE;
    gunsval[269][0][1] = TRUE;
    gunsval[269][1][0] = TRUE;
    gunsval[269][1][1] = TRUE;
    gunsval[269][1][3] = TRUE;
    gunsval[269][1][4] = TRUE;
    gunsval[269][2][0] = TRUE;
    gunsval[269][2][1] = TRUE;
    gunsval[269][3][1] = TRUE;
    gunsval[272][0][0] = TRUE;
    gunsval[272][0][1] = TRUE;
    gunsval[272][0][3] = TRUE;
    gunsval[272][1][0] = TRUE;
    gunsval[273][0][0] = TRUE;
    gunsval[273][0][1] = TRUE;
    gunsval[273][0][2] = TRUE;
    gunsval[273][0][3] = TRUE;
    gunsval[273][1][1] = TRUE;
    gunsval[273][1][2] = TRUE;
    gunsval[273][1][3] = TRUE;
    gunsval[274][0][0] = TRUE;
    gunsval[274][0][1] = TRUE;
    gunsval[277][0][0] = TRUE;
    gunsval[277][0][1] = TRUE;
    gunsval[278][0][0] = TRUE;
    gunsval[278][0][1] = TRUE;
    gunsval[280][0][0] = TRUE;
    gunsval[280][0][1] = TRUE;
    gunsval[283][0][0] = TRUE;
    gunsval[283][0][1] = TRUE;
    gunsval[283][0][3] = TRUE;
    gunsval[284][0][1] = TRUE;
    gunsval[284][0][2] = TRUE;
    gunsval[288][0][0] = TRUE;
    gunsval[288][0][1] = TRUE;
    gunsval[288][0][3] = TRUE;
    gunsval[288][0][4] = TRUE;
    gunsval[288][1][1] = TRUE;
    gunsval[289][0][0] = TRUE;
    gunsval[289][0][1] = TRUE;
    gunsval[289][0][3] = TRUE;
    gunsval[289][0][4] = TRUE;
    gunsval[289][1][1] = TRUE;
    gunsval[296][3][1] = TRUE;
    gunsval[297][3][1] = TRUE;
    gunsval[297][4][0] = TRUE;
    gunsval[297][4][1] = TRUE;
    gunsval[298][4][0] = TRUE;
    gunsval[299][2][1] = TRUE;
    gunsval[300][4][1] = TRUE;
    gunsval[304][5][0] = TRUE;
    gunsval[304][5][1] = TRUE;
    gunsval[304][5][3] = TRUE;
    gunsval[304][7][0] = TRUE;
    gunsval[304][7][1] = TRUE;
    gunsval[304][7][2] = TRUE;
    gunsval[304][7][3] = TRUE;
    gunsval[304][7][5] = TRUE;
    gunsval[304][7][6] = TRUE;
    gunsval[306][6][0] = TRUE;
    gunsval[306][7][0] = TRUE;
    gunsval[306][8][0] = TRUE;
    gunsval[306][8][1] = TRUE;
    gunsval[306][8][2] = TRUE;
    gunsval[306][8][4] = TRUE;
    gunsval[306][8][5] = TRUE;
    gunsval[306][8][6] = TRUE;
    gunsval[315][5][0] = TRUE;
    gunsval[315][5][1] = TRUE;
    gunsval[315][5][2] = TRUE;
    gunsval[315][5][4] = TRUE;
    gunsval[316][4][0] = TRUE;
    gunsval[316][4][1] = TRUE;
    gunsval[321][5][0] = TRUE;
    gunsval[321][5][1] = TRUE;
    gunsval[321][5][3] = TRUE;
    gunsval[321][8][1] = TRUE;
    gunsval[321][8][2] = TRUE;
    gunsval[322][0][0] = TRUE;
    gunsval[328][1][0] = TRUE;
    gunsval[328][1][1] = TRUE;
    gunsval[328][2][1] = TRUE;
    gunsval[328][2][2] = TRUE;
    gunsval[329][0][0] = TRUE;
    gunsval[329][0][1] = TRUE;
    gunsval[343][0][1] = TRUE;
    gunsval[343][5][0] = TRUE;
    gunsval[343][5][1] = TRUE;
    gunsval[343][5][3] = TRUE;
    gunsval[343][5][4] = TRUE;
    gunsval[343][5][5] = TRUE;
    gunsval[345][3][0] = TRUE;
    gunsval[345][3][1] = TRUE;
    gunsval[345][3][3] = TRUE;
    gunsval[345][3][4] = TRUE;
    gunsval[346][2][0] = TRUE;
    gunsval[346][2][1] = TRUE;
    gunsval[346][2][3] = TRUE;
    gunsval[347][8][0] = TRUE;
    gunsval[347][8][1] = TRUE;
    gunsval[347][8][2] = TRUE;
    gunsval[347][8][3] = TRUE;
    gunsval[347][8][4] = TRUE;
    gunsval[347][8][6] = TRUE;
    gunsval[347][8][7] = TRUE;
    gunsval[347][8][8] = TRUE;
    gunsval[348][0][0] = TRUE;
    gunsval[348][0][1] = TRUE;
    gunsval[348][0][2] = TRUE;
    gunsval[348][0][3] = TRUE;
    gunsval[348][0][5] = TRUE;
    gunsval[348][1][1] = TRUE;
    gunsval[348][2][1] = TRUE;
    gunsval[348][3][0] = TRUE;
    gunsval[348][3][1] = TRUE;
    gunsval[348][3][2] = TRUE;
    gunsval[349][5][1] = TRUE;
    gunsval[349][5][2] = TRUE;
    gunsval[349][5][3] = TRUE;
    gunsval[349][5][4] = TRUE;
    gunsval[349][5][5] = TRUE;
    gunsval[349][5][6] = TRUE;
    gunsval[349][5][7] = TRUE;
    gunsval[352][0][0] = TRUE;
    gunsval[352][0][1] = TRUE;
    gunsval[352][0][2] = TRUE;
    gunsval[352][0][3] = TRUE;
    gunsval[353][2][0] = TRUE;
    gunsval[353][2][2] = TRUE;
    gunsval[353][4][0] = TRUE;
    gunsval[353][4][1] = TRUE;
    gunsval[353][5][0] = TRUE;
    gunsval[353][5][2] = TRUE;
    gunsval[353][5][3] = TRUE;
    gunsval[353][5][4] = TRUE;
    gunsval[361][2][1] = TRUE;
    gunsval[361][3][0] = TRUE;
    gunsval[361][3][1] = TRUE;
    gunsval[361][3][2] = TRUE;
    gunsval[361][3][4] = TRUE;
    gunsval[365][1][1] = TRUE;
    gunsval[365][1][2] = TRUE;
    gunsval[365][3][0] = TRUE;
    gunsval[365][3][1] = TRUE;
    gunsval[365][3][3] = TRUE;
    gunsval[365][3][4] = TRUE;
    gunsval[365][4][1] = TRUE;
    gunsval[365][4][2] = TRUE;
    gunsval[366][0][1] = TRUE;
    gunsval[367][4][0] = TRUE;
    gunsval[367][4][1] = TRUE;
    gunsval[367][4][2] = TRUE;
    gunsval[367][4][4] = TRUE;
    gunsval[367][4][5] = TRUE;
    gunsval[367][4][6] = TRUE;
    gunsval[368][3][0] = TRUE;
    gunsval[368][3][1] = TRUE;
    gunsval[368][3][3] = TRUE;
    gunsval[368][3][4] = TRUE;
    gunsval[37][0][0] = TRUE;
    gunsval[37][1][1] = TRUE;
    gunsval[382][0][0] = TRUE;
    gunsval[383][1][0] = TRUE;
    gunsval[383][1][1] = TRUE;
    gunsval[383][1][2] = TRUE;
    gunsval[383][1][3] = TRUE;
    gunsval[383][1][5] = TRUE;
    gunsval[383][1][6] = TRUE;
    gunsval[383][1][7] = TRUE;
    gunsval[383][1][8] = TRUE;
    gunsval[384][0][1] = TRUE;
    gunsval[384][0][2] = TRUE;
    gunsval[384][2][0] = TRUE;
    gunsval[384][2][1] = TRUE;
    gunsval[384][2][2] = TRUE;
    gunsval[384][2][3] = TRUE;
    gunsval[384][2][5] = TRUE;
    gunsval[384][2][6] = TRUE;
    gunsval[384][2][7] = TRUE;
    gunsval[384][2][8] = TRUE;
    gunsval[384][5][0] = TRUE;
    gunsval[384][6][1] = TRUE;
    gunsval[38][0][0] = TRUE;
    gunsval[38][0][1] = TRUE;
    gunsval[38][0][2] = TRUE;
    gunsval[38][0][4] = TRUE;
    gunsval[38][1][1] = TRUE;
    gunsval[392][0][0] = TRUE;
    gunsval[392][0][1] = TRUE;
    gunsval[392][1][0] = TRUE;
    gunsval[392][1][1] = TRUE;
    gunsval[393][0][0] = TRUE;
    gunsval[393][0][1] = TRUE;
    gunsval[393][0][3] = TRUE;
    gunsval[393][0][4] = TRUE;
    gunsval[393][2][0] = TRUE;
    gunsval[393][4][0] = TRUE;
    gunsval[393][4][2] = TRUE;
    gunsval[393][4][3] = TRUE;
    gunsval[394][0][0] = TRUE;
    gunsval[394][1][0] = TRUE;
    gunsval[394][1][1] = TRUE;
    gunsval[394][1][2] = TRUE;
    gunsval[394][2][0] = TRUE;
    gunsval[394][2][2] = TRUE;
    gunsval[394][2][3] = TRUE;
    gunsval[394][2][4] = TRUE;
    gunsval[395][1][0] = TRUE;
    gunsval[395][2][1] = TRUE;
    gunsval[395][2][2] = TRUE;
    gunsval[395][2][3] = TRUE;
    gunsval[395][4][2] = TRUE;
    gunsval[395][5][1] = TRUE;
    gunsval[395][6][1] = TRUE;
    gunsval[397][0][0] = TRUE;
    gunsval[397][2][0] = TRUE;
    gunsval[397][2][1] = TRUE;
    gunsval[397][2][3] = TRUE;
    gunsval[397][2][4] = TRUE;
    gunsval[398][0][1] = TRUE;
    gunsval[398][3][0] = TRUE;
    gunsval[398][3][2] = TRUE;
    gunsval[398][3][3] = TRUE;
    gunsval[398][3][4] = TRUE;
    gunsval[398][4][1] = TRUE;
    gunsval[398][5][1] = TRUE;
    gunsval[399][3][0] = TRUE;
    gunsval[399][3][1] = TRUE;
    gunsval[399][3][2] = TRUE;
    gunsval[399][3][4] = TRUE;
    gunsval[3][6][0] = TRUE;
    gunsval[3][6][1] = TRUE;
    gunsval[3][6][2] = TRUE;
    gunsval[3][6][3] = TRUE;
    gunsval[3][6][4] = TRUE;
    gunsval[3][6][5] = TRUE;
    gunsval[40][0][0] = TRUE;
    gunsval[40][0][1] = TRUE;
    gunsval[40][0][2] = TRUE;
    gunsval[40][0][4] = TRUE;
    gunsval[40][1][0] = TRUE;
    gunsval[40][1][2] = TRUE;
    gunsval[41][0][0] = TRUE;
    gunsval[41][1][1] = TRUE;
    gunsval[41][2][1] = TRUE;
    gunsval[41][2][2] = TRUE;
    gunsval[42][0][0] = TRUE;
    gunsval[42][0][1] = TRUE;
    gunsval[42][0][2] = TRUE;
    gunsval[42][0][4] = TRUE;
    gunsval[42][1][0] = TRUE;
    gunsval[42][1][1] = TRUE;
    gunsval[43][0][0] = TRUE;
    gunsval[43][1][1] = TRUE;
    gunsval[43][1][2] = TRUE;
    gunsval[44][0][0] = TRUE;
    gunsval[44][0][1] = TRUE;
    gunsval[44][0][2] = TRUE;
    gunsval[44][0][4] = TRUE;
    gunsval[46][1][1] = TRUE;
    gunsval[46][1][2] = TRUE;
    gunsval[46][1][3] = TRUE;
    gunsval[46][2][0] = TRUE;
    gunsval[46][2][1] = TRUE;
    gunsval[46][2][3] = TRUE;
    gunsval[48][1][0] = TRUE;
    gunsval[48][1][1] = TRUE;
    gunsval[48][1][3] = TRUE;
    gunsval[48][2][0] = TRUE;
    gunsval[48][2][1] = TRUE;
    gunsval[48][2][3] = TRUE;
    gunsval[4][6][0] = TRUE;
    gunsval[4][6][1] = TRUE;
    gunsval[4][6][2] = TRUE;
    gunsval[4][6][3] = TRUE;
    gunsval[4][6][4] = TRUE;
    gunsval[4][6][5] = TRUE;
    gunsval[54][1][1] = TRUE;
    gunsval[54][1][2] = TRUE;
    gunsval[54][1][3] = TRUE;
    gunsval[59][1][2] = TRUE;
    gunsval[59][1][3] = TRUE;
    gunsval[59][4][0] = TRUE;
    gunsval[59][4][2] = TRUE;
    gunsval[59][4][3] = TRUE;
    gunsval[59][4][4] = TRUE;
    gunsval[59][4][5] = TRUE;
    gunsval[60][2][3] = TRUE;
    gunsval[60][2][4] = TRUE;
    gunsval[60][4][1] = TRUE;
    gunsval[60][4][2] = TRUE;
    gunsval[60][4][3] = TRUE;
    gunsval[60][4][4] = TRUE;
    gunsval[60][4][5] = TRUE;
    gunsval[63][3][0] = TRUE;
    gunsval[63][3][1] = TRUE;
    gunsval[63][3][3] = TRUE;
    gunsval[63][3][4] = TRUE;
    gunsval[71][1][1] = TRUE;
    gunsval[71][1][2] = TRUE;
    gunsval[71][1][3] = TRUE;
    gunsval[71][2][0] = TRUE;
    gunsval[71][2][1] = TRUE;
    gunsval[71][2][3] = TRUE;
    gunsval[74][0][1] = TRUE;
    gunsval[74][0][2] = TRUE;
    gunsval[74][0][3] = TRUE;
    gunsval[74][2][0] = TRUE;
    gunsval[74][2][1] = TRUE;
    gunsval[74][2][3] = TRUE;
    gunsval[82][0][2] = TRUE;
    gunsval[84][0][3] = TRUE;
    gunsval[85][0][3] = TRUE;
    gunsval[86][0][3] = TRUE;
    gunsval[87][0][3] = TRUE;
    gunsval[88][1][2] = TRUE;
    gunsval[89][2][2] = TRUE;
    gunsval[90][1][0] = TRUE;
    gunsval[92][1][2] = TRUE;
    gunsval[93][5][0] = TRUE;
    gunsval[93][5][1] = TRUE;
    gunsval[93][5][2] = TRUE;
    gunsval[93][5][4] = TRUE;
    gunsval[94][5][0] = TRUE;
    gunsval[94][5][1] = TRUE;
    gunsval[94][5][2] = TRUE;
    gunsval[94][5][4] = TRUE;
    gunsval[94][5][5] = TRUE;
    gunsval[94][5][6] = TRUE;
    gunsval[95][1][0] = TRUE;
    gunsval[95][1][1] = TRUE;
    gunsval[95][1][3] = TRUE;
    gunsval[95][2][1] = TRUE;
    gunsval[96][1][0] = TRUE;
    gunsval[96][1][10] = TRUE;
    gunsval[96][1][2] = TRUE;
    gunsval[96][1][3] = TRUE;
    gunsval[96][1][4] = TRUE;
    gunsval[96][1][5] = TRUE;
    gunsval[96][1][6] = TRUE;
    gunsval[96][1][7] = TRUE;
    gunsval[96][1][8] = TRUE;
    gunsval[96][1][9] = TRUE;
    gunsval[96][3][0] = TRUE;
    gunsval[96][3][1] = TRUE;
    gunsval[96][3][10] = TRUE;
    gunsval[96][3][11] = TRUE;
    gunsval[96][3][2] = TRUE;
    gunsval[96][3][3] = TRUE;
    gunsval[96][3][5] = TRUE;
    gunsval[96][3][6] = TRUE;
    gunsval[96][3][7] = TRUE;
    gunsval[96][3][8] = TRUE;
    gunsval[96][3][9] = TRUE;
    gunsval[97][0][1] = TRUE;
    gunsval[97][1][0] = TRUE;
    gunsval[97][1][1] = TRUE;
    gunsval[97][1][2] = TRUE;
    gunsval[97][2][0] = TRUE;
    gunsval[97][3][0] = TRUE;
    gunsval[97][3][1] = TRUE;
    gunsval[97][4][0] = TRUE;
    gunsval[97][4][2] = TRUE;
    gunsval[97][5][0] = TRUE;
    gunsval[97][5][2] = TRUE;
    gunsval[97][5][3] = TRUE;
    gunsval[98][2][0] = TRUE;
    gunsval[98][2][1] = TRUE;
    gunsval[98][3][1] = TRUE;
    gunsval[98][3][2] = TRUE;
  }



  if ( gunslevel == 2 ) {
    gunsval[0][5][0] = TRUE;
    gunsval[0][5][1] = TRUE;
    gunsval[0][5][2] = TRUE;
    gunsval[0][5][3] = TRUE;
    gunsval[0][5][4] = TRUE;
    gunsval[0][5][5] = TRUE;
    gunsval[100][1][0] = TRUE;
    gunsval[100][2][0] = TRUE;
    gunsval[101][0][2] = TRUE;
    gunsval[101][1][0] = TRUE;
    gunsval[101][1][10] = TRUE;
    gunsval[101][1][2] = TRUE;
    gunsval[101][1][3] = TRUE;
    gunsval[101][1][4] = TRUE;
    gunsval[101][1][5] = TRUE;
    gunsval[101][1][6] = TRUE;
    gunsval[101][1][7] = TRUE;
    gunsval[101][1][8] = TRUE;
    gunsval[101][1][9] = TRUE;
    gunsval[101][2][0] = TRUE;
    gunsval[101][2][1] = TRUE;
    gunsval[101][3][0] = TRUE;
    gunsval[101][3][1] = TRUE;
    gunsval[101][3][10] = TRUE;
    gunsval[101][3][11] = TRUE;
    gunsval[101][3][2] = TRUE;
    gunsval[101][3][3] = TRUE;
    gunsval[101][3][4] = TRUE;
    gunsval[101][3][5] = TRUE;
    gunsval[101][3][7] = TRUE;
    gunsval[101][3][8] = TRUE;
    gunsval[101][3][9] = TRUE;
    gunsval[102][1][1] = TRUE;
    gunsval[102][1][2] = TRUE;
    gunsval[102][1][3] = TRUE;
    gunsval[102][1][4] = TRUE;
    gunsval[102][1][5] = TRUE;
    gunsval[102][1][6] = TRUE;
    gunsval[104][6][1] = TRUE;
    gunsval[104][6][2] = TRUE;
    gunsval[106][1][0] = TRUE;
    gunsval[106][1][1] = TRUE;
    gunsval[106][1][3] = TRUE;
    gunsval[106][4][1] = TRUE;
    gunsval[106][4][10] = TRUE;
    gunsval[106][4][2] = TRUE;
    gunsval[106][4][3] = TRUE;
    gunsval[106][4][4] = TRUE;
    gunsval[106][4][5] = TRUE;
    gunsval[106][4][6] = TRUE;
    gunsval[106][4][7] = TRUE;
    gunsval[106][4][8] = TRUE;
    gunsval[106][4][9] = TRUE;
    gunsval[106][6][1] = TRUE;
    gunsval[106][6][2] = TRUE;
    gunsval[107][3][0] = TRUE;
    gunsval[107][3][1] = TRUE;
    gunsval[107][3][2] = TRUE;
    gunsval[107][6][0] = TRUE;
    gunsval[107][6][1] = TRUE;
    gunsval[107][6][3] = TRUE;
    gunsval[107][6][4] = TRUE;
    gunsval[107][6][5] = TRUE;
    gunsval[107][7][0] = TRUE;
    gunsval[107][7][2] = TRUE;
    gunsval[108][0][0] = TRUE;
    gunsval[108][0][1] = TRUE;
    gunsval[108][0][3] = TRUE;
    gunsval[108][0][4] = TRUE;
    gunsval[108][0][5] = TRUE;
    gunsval[108][0][6] = TRUE;
    gunsval[108][1][0] = TRUE;
    gunsval[108][1][1] = TRUE;
    gunsval[108][1][2] = TRUE;
    gunsval[108][1][4] = TRUE;
    gunsval[108][1][5] = TRUE;
    gunsval[108][1][6] = TRUE;
    gunsval[109][1][1] = TRUE;
    gunsval[109][1][2] = TRUE;
    gunsval[109][1][3] = TRUE;
    gunsval[109][1][4] = TRUE;
    gunsval[109][1][5] = TRUE;
    gunsval[109][1][6] = TRUE;
    gunsval[109][1][7] = TRUE;
    gunsval[109][1][8] = TRUE;
    gunsval[109][2][0] = TRUE;
    gunsval[109][2][1] = TRUE;
    gunsval[109][2][2] = TRUE;
    gunsval[109][2][3] = TRUE;
    gunsval[109][2][5] = TRUE;
    gunsval[109][2][6] = TRUE;
    gunsval[109][2][7] = TRUE;
    gunsval[109][2][8] = TRUE;
    gunsval[10][1][0] = TRUE;
    gunsval[10][2][0] = TRUE;
    gunsval[10][2][1] = TRUE;
    gunsval[113][7][0] = TRUE;
    gunsval[113][7][1] = TRUE;
    gunsval[113][7][2] = TRUE;
    gunsval[113][7][3] = TRUE;
    gunsval[113][7][5] = TRUE;
    gunsval[113][7][6] = TRUE;
    gunsval[115][1][0] = TRUE;
    gunsval[115][2][1] = TRUE;
    gunsval[115][2][2] = TRUE;
    gunsval[115][4][0] = TRUE;
    gunsval[117][2][0] = TRUE;
    gunsval[117][2][1] = TRUE;
    gunsval[117][2][2] = TRUE;
    gunsval[117][2][3] = TRUE;
    gunsval[117][2][4] = TRUE;
    gunsval[117][2][5] = TRUE;
    gunsval[117][2][6] = TRUE;
    gunsval[117][2][7] = TRUE;
    gunsval[117][2][8] = TRUE;
    gunsval[117][2][9] = TRUE;
    gunsval[117][3][0] = TRUE;
    gunsval[117][3][2] = TRUE;
    gunsval[117][4][1] = TRUE;
    gunsval[117][4][2] = TRUE;
    gunsval[117][5][0] = TRUE;
    gunsval[117][5][2] = TRUE;
    gunsval[118][0][0] = TRUE;
    gunsval[118][0][1] = TRUE;
    gunsval[118][0][3] = TRUE;
    gunsval[118][1][0] = TRUE;
    gunsval[118][1][2] = TRUE;
    gunsval[118][1][3] = TRUE;
    gunsval[118][1][4] = TRUE;
    gunsval[118][1][5] = TRUE;
    gunsval[118][1][6] = TRUE;
    gunsval[118][1][7] = TRUE;
    gunsval[118][1][8] = TRUE;
    gunsval[119][8][0] = TRUE;
    gunsval[119][8][1] = TRUE;
    gunsval[119][8][2] = TRUE;
    gunsval[119][8][4] = TRUE;
    gunsval[119][8][5] = TRUE;
    gunsval[119][8][6] = TRUE;
    gunsval[119][8][7] = TRUE;
    gunsval[119][8][8] = TRUE;
    gunsval[119][8][9] = TRUE;
    gunsval[120][10][0] = TRUE;
    gunsval[120][10][1] = TRUE;
    gunsval[120][10][2] = TRUE;
    gunsval[120][10][3] = TRUE;
    gunsval[120][10][4] = TRUE;
    gunsval[120][10][6] = TRUE;
    gunsval[120][10][7] = TRUE;
    gunsval[120][10][8] = TRUE;
    gunsval[120][10][9] = TRUE;
    gunsval[120][1][0] = TRUE;
    gunsval[120][4][0] = TRUE;
    gunsval[120][4][1] = TRUE;
    gunsval[120][4][2] = TRUE;
    gunsval[120][5][0] = TRUE;
    gunsval[120][5][1] = TRUE;
    gunsval[120][5][2] = TRUE;
    gunsval[120][6][0] = TRUE;
    gunsval[120][6][2] = TRUE;
    gunsval[120][6][3] = TRUE;
    gunsval[120][8][0] = TRUE;
    gunsval[120][8][1] = TRUE;
    gunsval[120][8][2] = TRUE;
    gunsval[120][8][3] = TRUE;
    gunsval[120][8][5] = TRUE;
    gunsval[120][8][6] = TRUE;
    gunsval[120][8][7] = TRUE;
    gunsval[120][8][8] = TRUE;
    gunsval[120][9][0] = TRUE;
    gunsval[120][9][2] = TRUE;
    gunsval[120][9][3] = TRUE;
    gunsval[121][5][0] = TRUE;
    gunsval[121][5][1] = TRUE;
    gunsval[121][5][2] = TRUE;
    gunsval[121][5][3] = TRUE;
    gunsval[121][5][4] = TRUE;
    gunsval[121][5][6] = TRUE;
    gunsval[121][5][7] = TRUE;
    gunsval[121][5][8] = TRUE;
    gunsval[121][5][9] = TRUE;
    gunsval[121][6][0] = TRUE;
    gunsval[121][6][1] = TRUE;
    gunsval[125][5][0] = TRUE;
    gunsval[125][5][1] = TRUE;
    gunsval[125][5][3] = TRUE;
    gunsval[125][5][4] = TRUE;
    gunsval[126][2][0] = TRUE;
    gunsval[126][2][1] = TRUE;
    gunsval[126][2][3] = TRUE;
    gunsval[126][2][4] = TRUE;
    gunsval[127][1][2] = TRUE;
    gunsval[127][1][3] = TRUE;
    gunsval[127][1][4] = TRUE;
    gunsval[128][0][0] = TRUE;
    gunsval[128][0][1] = TRUE;
    gunsval[128][0][2] = TRUE;
    gunsval[129][3][0] = TRUE;
  gunsval[129][3][1] = TRUE;
  gunsval[129][3][2] = TRUE;
  gunsval[12][2][0] = TRUE;
  gunsval[12][2][2] = TRUE;
  gunsval[12][3][1] = TRUE;
  gunsval[12][3][2] = TRUE;
  gunsval[12][3][3] = TRUE;
  gunsval[136][2][0] = TRUE;
  gunsval[136][2][1] = TRUE;
  gunsval[137][6][0] = TRUE;
  gunsval[138][0][0] = TRUE;
  gunsval[138][0][1] = TRUE;
  gunsval[138][3][0] = TRUE;
  gunsval[138][3][1] = TRUE;
  gunsval[138][3][2] = TRUE;
  gunsval[138][3][3] = TRUE;
  gunsval[138][3][5] = TRUE;
  gunsval[138][3][6] = TRUE;
  gunsval[138][6][0] = TRUE;
  gunsval[138][6][1] = TRUE;
  gunsval[139][2][0] = TRUE;
  gunsval[139][2][1] = TRUE;
  gunsval[139][5][0] = TRUE;
  gunsval[139][5][2] = TRUE;
  gunsval[139][6][0] = TRUE;
  gunsval[139][6][1] = TRUE;
  gunsval[139][7][1] = TRUE;
  gunsval[139][7][2] = TRUE;
  gunsval[139][7][3] = TRUE;
  gunsval[13][0][0] = TRUE;
  gunsval[13][1][0] = TRUE;
  gunsval[13][1][2] = TRUE;
  gunsval[141][10][0] = TRUE;
  gunsval[141][10][2] = TRUE;
  gunsval[141][10][3] = TRUE;
  gunsval[141][11][0] = TRUE;
  gunsval[141][11][2] = TRUE;
  gunsval[141][11][3] = TRUE;
  gunsval[141][12][0] = TRUE;
  gunsval[141][12][1] = TRUE;
  gunsval[141][13][0] = TRUE;
  gunsval[141][13][2] = TRUE;
  gunsval[141][13][3] = TRUE;
  gunsval[141][14][1] = TRUE;
  gunsval[141][14][2] = TRUE;
  gunsval[141][14][3] = TRUE;
  gunsval[141][2][0] = TRUE;
  gunsval[141][2][1] = TRUE;
  gunsval[141][8][1] = TRUE;
  gunsval[141][8][2] = TRUE;
  gunsval[141][9][0] = TRUE;
  gunsval[141][9][2] = TRUE;
  gunsval[142][4][0] = TRUE;
  gunsval[142][4][2] = TRUE;
  gunsval[142][4][3] = TRUE;
  gunsval[142][9][0] = TRUE;
  gunsval[142][9][2] = TRUE;
  gunsval[143][3][0] = TRUE;
  gunsval[143][3][1] = TRUE;
  gunsval[144][10][0] = TRUE;
  gunsval[144][10][2] = TRUE;
  gunsval[144][7][0] = TRUE;
  gunsval[144][7][2] = TRUE;
  gunsval[144][7][3] = TRUE;
  gunsval[145][1][0] = TRUE;
  gunsval[145][1][1] = TRUE;
  gunsval[145][5][0] = TRUE;
  gunsval[145][5][1] = TRUE;
  gunsval[145][6][0] = TRUE;
  gunsval[145][6][2] = TRUE;
  gunsval[145][7][0] = TRUE;
  gunsval[145][7][1] = TRUE;
  gunsval[145][8][0] = TRUE;
  gunsval[145][8][1] = TRUE;
  gunsval[146][10][0] = TRUE;
  gunsval[146][10][1] = TRUE;
  gunsval[146][11][1] = TRUE;
  gunsval[146][11][2] = TRUE;
  gunsval[146][12][1] = TRUE;
  gunsval[146][12][2] = TRUE;
  gunsval[146][12][3] = TRUE;
  gunsval[146][1][0] = TRUE;
  gunsval[146][1][1] = TRUE;
  gunsval[146][1][2] = TRUE;
  gunsval[146][5][0] = TRUE;
  gunsval[146][5][1] = TRUE;
  gunsval[146][7][0] = TRUE;
  gunsval[146][7][2] = TRUE;
  gunsval[146][8][0] = TRUE;
  gunsval[146][8][2] = TRUE;
  gunsval[146][8][3] = TRUE;
  gunsval[146][9][1] = TRUE;
  gunsval[146][9][2] = TRUE;
  gunsval[146][9][3] = TRUE;
  gunsval[147][11][0] = TRUE;
  gunsval[147][11][2] = TRUE;
  gunsval[147][6][0] = TRUE;
  gunsval[147][6][2] = TRUE;
  gunsval[147][6][3] = TRUE;
  gunsval[148][10][0] = TRUE;
  gunsval[148][10][2] = TRUE;
  gunsval[148][10][3] = TRUE;
  gunsval[148][12][0] = TRUE;
  gunsval[148][12][2] = TRUE;
  gunsval[149][3][0] = TRUE;
  gunsval[149][3][2] = TRUE;
  gunsval[149][4][0] = TRUE;
  gunsval[149][4][1] = TRUE;
  gunsval[149][5][0] = TRUE;
  gunsval[149][5][1] = TRUE;
  gunsval[149][6][0] = TRUE;
  gunsval[149][6][1] = TRUE;
  gunsval[149][6][2] = TRUE;
  gunsval[14][1][0] = TRUE;
  gunsval[14][2][0] = TRUE;
  gunsval[14][2][1] = TRUE;
  gunsval[157][3][1] = TRUE;
  gunsval[157][3][2] = TRUE;
  gunsval[157][4][0] = TRUE;
  gunsval[157][4][1] = TRUE;
  gunsval[157][4][3] = TRUE;
  gunsval[157][4][4] = TRUE;
  gunsval[157][5][0] = TRUE;
  gunsval[157][5][2] = TRUE;
  gunsval[157][6][1] = TRUE;
  gunsval[157][6][2] = TRUE;
  gunsval[159][3][1] = TRUE;
  gunsval[159][3][2] = TRUE;
  gunsval[159][4][0] = TRUE;
  gunsval[159][4][1] = TRUE;
  gunsval[159][5][0] = TRUE;
  gunsval[15][0][0] = TRUE;
  gunsval[15][1][0] = TRUE;
  gunsval[15][1][1] = TRUE;
  gunsval[167][0][0] = TRUE;
  gunsval[167][0][2] = TRUE;
  gunsval[168][1][0] = TRUE;
  gunsval[168][1][1] = TRUE;
  gunsval[168][3][0] = TRUE;
  gunsval[168][3][1] = TRUE;
  gunsval[168][3][3] = TRUE;
  gunsval[176][1][1] = TRUE;
  gunsval[176][2][0] = TRUE;
  gunsval[176][2][1] = TRUE;
  gunsval[179][1][0] = TRUE;
  gunsval[17][1][0] = TRUE;
  gunsval[17][1][1] = TRUE;
  gunsval[17][1][3] = TRUE;
  gunsval[17][2][0] = TRUE;
  gunsval[17][2][2] = TRUE;
  gunsval[181][2][0] = TRUE;
  gunsval[181][2][1] = TRUE;
  gunsval[181][2][3] = TRUE;
  gunsval[182][0][0] = TRUE;
  gunsval[182][0][1] = TRUE;
  gunsval[182][0][2] = TRUE;
  gunsval[182][0][3] = TRUE;
  gunsval[182][1][0] = TRUE;
  gunsval[182][1][1] = TRUE;
  gunsval[182][2][0] = TRUE;
  gunsval[182][2][1] = TRUE;
  gunsval[182][2][2] = TRUE;
  gunsval[182][2][3] = TRUE;
  gunsval[182][2][4] = TRUE;
  gunsval[182][2][5] = TRUE;
  gunsval[183][0][0] = TRUE;
  gunsval[183][0][2] = TRUE;
  gunsval[183][1][0] = TRUE;
  gunsval[183][1][1] = TRUE;
  gunsval[183][2][0] = TRUE;
  gunsval[183][2][1] = TRUE;
  gunsval[183][2][2] = TRUE;
  gunsval[183][2][4] = TRUE;
  gunsval[184][2][0] = TRUE;
  gunsval[184][2][1] = TRUE;
  gunsval[184][2][2] = TRUE;
  gunsval[184][2][3] = TRUE;
  gunsval[184][2][4] = TRUE;
  gunsval[184][2][6] = TRUE;
  gunsval[187][2][0] = TRUE;
  gunsval[187][4][0] = TRUE;
  gunsval[187][4][1] = TRUE;
  gunsval[187][4][2] = TRUE;
  gunsval[187][4][4] = TRUE;
  gunsval[188][0][0] = TRUE;
  gunsval[188][1][0] = TRUE;
  gunsval[188][1][1] = TRUE;
  gunsval[188][1][3] = TRUE;
  gunsval[188][3][0] = TRUE;
  gunsval[188][3][1] = TRUE;
  gunsval[188][3][2] = TRUE;
  gunsval[188][3][4] = TRUE;
  gunsval[189][0][0] = TRUE;
  gunsval[189][1][0] = TRUE;
  gunsval[189][2][0] = TRUE;
  gunsval[189][2][1] = TRUE;
  gunsval[189][2][2] = TRUE;
  gunsval[189][4][0] = TRUE;
  gunsval[189][4][1] = TRUE;
  gunsval[189][4][2] = TRUE;
  gunsval[189][4][3] = TRUE;
  gunsval[189][4][5] = TRUE;
  gunsval[18][1][0] = TRUE;
  gunsval[18][1][1] = TRUE;
  gunsval[18][1][2] = TRUE;
  gunsval[18][3][0] = TRUE;
  gunsval[18][3][1] = TRUE;
  gunsval[190][1][0] = TRUE;
  gunsval[190][3][0] = TRUE;
  gunsval[190][3][1] = TRUE;
  gunsval[190][3][3] = TRUE;
  gunsval[191][1][0] = TRUE;
  gunsval[191][1][1] = TRUE;
  gunsval[192][0][2] = TRUE;
  gunsval[192][1][0] = TRUE;
  gunsval[192][2][0] = TRUE;
  gunsval[192][3][0] = TRUE;
  gunsval[192][4][0] = TRUE;
  gunsval[192][4][2] = TRUE;
  gunsval[192][4][3] = TRUE;
  gunsval[192][4][4] = TRUE;
  gunsval[193][0][1] = TRUE;
  gunsval[193][0][2] = TRUE;
  gunsval[193][2][1] = TRUE;
  gunsval[193][2][2] = TRUE;
  gunsval[194][1][1] = TRUE;
  gunsval[198][3][1] = TRUE;
  gunsval[198][3][2] = TRUE;
  gunsval[198][3][3] = TRUE;
  gunsval[19][1][0] = TRUE;
  gunsval[19][1][1] = TRUE;
  gunsval[19][1][3] = TRUE;
  gunsval[19][2][0] = TRUE;
  gunsval[19][2][1] = TRUE;
  gunsval[1][0][0] = TRUE;
  gunsval[1][0][1] = TRUE;
  gunsval[1][6][0] = TRUE;
  gunsval[1][6][1] = TRUE;
  gunsval[1][6][2] = TRUE;
  gunsval[1][6][3] = TRUE;
  gunsval[1][6][4] = TRUE;
  gunsval[1][6][5] = TRUE;
  gunsval[200][2][0] = TRUE;
  gunsval[200][2][1] = TRUE;
  gunsval[205][0][0] = TRUE;
  gunsval[205][0][2] = TRUE;
  gunsval[205][1][1] = TRUE;
  gunsval[207][2][0] = TRUE;
  gunsval[207][2][1] = TRUE;
  gunsval[207][2][3] = TRUE;
  gunsval[207][2][4] = TRUE;
  gunsval[208][0][1] = TRUE;
  gunsval[208][0][2] = TRUE;
  gunsval[209][4][0] = TRUE;
  gunsval[209][4][1] = TRUE;
  gunsval[209][4][2] = TRUE;
  gunsval[209][4][4] = TRUE;
  gunsval[209][6][0] = TRUE;
  gunsval[209][6][1] = TRUE;
  gunsval[210][5][0] = TRUE;
  gunsval[210][5][1] = TRUE;
  gunsval[210][5][3] = TRUE;
  gunsval[210][5][4] = TRUE;
  gunsval[210][6][0] = TRUE;
  gunsval[210][6][1] = TRUE;
  gunsval[219][5][0] = TRUE;
  gunsval[219][5][1] = TRUE;
  gunsval[21][2][0] = TRUE;
  gunsval[220][4][0] = TRUE;
  gunsval[220][4][1] = TRUE;
  gunsval[220][4][2] = TRUE;
  gunsval[220][4][3] = TRUE;
  gunsval[220][4][4] = TRUE;
  gunsval[220][4][5] = TRUE;
  gunsval[220][4][7] = TRUE;
  gunsval[220][4][8] = TRUE;
  gunsval[221][4][0] = TRUE;
  gunsval[221][4][1] = TRUE;
  gunsval[221][4][2] = TRUE;
  gunsval[221][4][3] = TRUE;
  gunsval[221][4][4] = TRUE;
  gunsval[221][4][5] = TRUE;
  gunsval[221][4][7] = TRUE;
  gunsval[221][4][8] = TRUE;
  gunsval[222][2][0] = TRUE;
  gunsval[223][4][0] = TRUE;
  gunsval[223][4][1] = TRUE;
  gunsval[223][4][2] = TRUE;
  gunsval[223][4][3] = TRUE;
  gunsval[223][4][4] = TRUE;
  gunsval[223][4][6] = TRUE;
  gunsval[224][1][0] = TRUE;
  gunsval[224][1][1] = TRUE;
  gunsval[238][0][0] = TRUE;
  gunsval[238][0][2] = TRUE;
  gunsval[238][0][3] = TRUE;
  gunsval[238][0][4] = TRUE;
  gunsval[238][1][0] = TRUE;
  gunsval[238][1][1] = TRUE;
  gunsval[238][1][3] = TRUE;
  gunsval[238][1][4] = TRUE;
  gunsval[240][0][1] = TRUE;
  gunsval[240][0][2] = TRUE;
  gunsval[240][0][3] = TRUE;
  gunsval[240][0][4] = TRUE;
  gunsval[246][4][0] = TRUE;
  gunsval[246][6][0] = TRUE;
  gunsval[247][3][0] = TRUE;
  gunsval[248][0][0] = TRUE;
  gunsval[248][2][0] = TRUE;
  gunsval[248][2][2] = TRUE;
  gunsval[248][3][0] = TRUE;
  gunsval[248][4][0] = TRUE;
  gunsval[248][4][2] = TRUE;
  gunsval[248][5][0] = TRUE;
  gunsval[249][2][0] = TRUE;
  gunsval[249][2][2] = TRUE;
  gunsval[249][4][0] = TRUE;
  gunsval[249][5][0] = TRUE;
  gunsval[249][5][1] = TRUE;
  gunsval[249][5][2] = TRUE;
  gunsval[249][5][4] = TRUE;
  gunsval[249][5][5] = TRUE;
  gunsval[249][6][1] = TRUE;
  gunsval[250][0][0] = TRUE;
  gunsval[250][2][0] = TRUE;
  gunsval[250][2][1] = TRUE;
  gunsval[250][3][0] = TRUE;
  gunsval[250][4][1] = TRUE;
  gunsval[250][5][0] = TRUE;
  gunsval[250][6][0] = TRUE;
  gunsval[250][6][2] = TRUE;
  gunsval[251][0][0] = TRUE;
  gunsval[251][1][0] = TRUE;
  gunsval[251][1][2] = TRUE;
  gunsval[251][2][1] = TRUE;
  gunsval[251][2][2] = TRUE;
  gunsval[251][4][0] = TRUE;
  gunsval[251][5][0] = TRUE;
  gunsval[251][6][0] = TRUE;
  gunsval[251][6][2] = TRUE;
  gunsval[251][7][1] = TRUE;
  gunsval[251][7][3] = TRUE;
  gunsval[252][2][0] = TRUE;
  gunsval[252][4][0] = TRUE;
  gunsval[252][5][0] = TRUE;
  gunsval[254][0][0] = TRUE;
  gunsval[254][5][0] = TRUE;
  gunsval[256][0][1] = TRUE;
  gunsval[256][0][2] = TRUE;
  gunsval[256][1][0] = TRUE;
  gunsval[256][1][3] = TRUE;
  gunsval[256][2][0] = TRUE;
  gunsval[256][2][1] = TRUE;
  gunsval[256][3][0] = TRUE;
  gunsval[256][3][1] = TRUE;
  gunsval[256][5][0] = TRUE;
  gunsval[259][1][1] = TRUE;
  gunsval[259][1][2] = TRUE;
  gunsval[259][2][0] = TRUE;
  gunsval[259][3][0] = TRUE;
  gunsval[259][3][1] = TRUE;
  gunsval[259][3][3] = TRUE;
  gunsval[260][3][0] = TRUE;
  gunsval[261][0][0] = TRUE;
  gunsval[261][1][0] = TRUE;
  gunsval[261][1][1] = TRUE;
  gunsval[261][2][0] = TRUE;
  gunsval[261][2][1] = TRUE;
  gunsval[261][2][3] = TRUE;
  gunsval[261][2][4] = TRUE;
  gunsval[261][3][1] = TRUE;
  gunsval[261][3][2] = TRUE;
  gunsval[262][3][0] = TRUE;
  gunsval[262][3][1] = TRUE;
  gunsval[264][0][0] = TRUE;
  gunsval[264][0][1] = TRUE;
  gunsval[264][1][0] = TRUE;
  gunsval[264][1][4] = TRUE;
  gunsval[264][2][0] = TRUE;
  gunsval[264][2][1] = TRUE;
  gunsval[265][0][0] = TRUE;
  gunsval[265][0][1] = TRUE;
  gunsval[265][0][2] = TRUE;
  gunsval[265][1][0] = TRUE;
  gunsval[265][1][1] = TRUE;
  gunsval[265][1][3] = TRUE;
  gunsval[265][1][4] = TRUE;
  gunsval[265][2][0] = TRUE;
  gunsval[267][0][0] = TRUE;
  gunsval[267][0][1] = TRUE;
  gunsval[267][1][0] = TRUE;
  gunsval[267][1][1] = TRUE;
  gunsval[267][1][4] = TRUE;
  gunsval[267][2][0] = TRUE;
  gunsval[267][2][1] = TRUE;
  gunsval[268][0][1] = TRUE;
  gunsval[268][0][2] = TRUE;
  gunsval[268][1][0] = TRUE;
  gunsval[268][2][0] = TRUE;
  gunsval[268][2][1] = TRUE;
  gunsval[269][0][0] = TRUE;
  gunsval[269][0][1] = TRUE;
  gunsval[269][1][0] = TRUE;
  gunsval[269][1][1] = TRUE;
  gunsval[269][1][3] = TRUE;
  gunsval[269][1][4] = TRUE;
  gunsval[269][2][0] = TRUE;
  gunsval[269][2][1] = TRUE;
  gunsval[269][3][1] = TRUE;
  gunsval[272][0][0] = TRUE;
  gunsval[272][0][1] = TRUE;
  gunsval[272][0][3] = TRUE;
  gunsval[272][1][0] = TRUE;
  gunsval[273][0][0] = TRUE;
  gunsval[273][0][1] = TRUE;
  gunsval[273][0][2] = TRUE;
  gunsval[273][0][3] = TRUE;
  gunsval[273][1][1] = TRUE;
  gunsval[273][1][2] = TRUE;
  gunsval[273][1][3] = TRUE;
  gunsval[274][0][0] = TRUE;
  gunsval[274][0][1] = TRUE;
  gunsval[277][0][0] = TRUE;
  gunsval[277][0][1] = TRUE;
  gunsval[278][0][0] = TRUE;
  gunsval[278][0][1] = TRUE;
  gunsval[280][0][0] = TRUE;
  gunsval[280][0][1] = TRUE;
  gunsval[283][0][0] = TRUE;
  gunsval[283][0][1] = TRUE;
  gunsval[283][0][3] = TRUE;
  gunsval[284][0][1] = TRUE;
  gunsval[284][0][2] = TRUE;
  gunsval[288][0][0] = TRUE;
  gunsval[288][0][1] = TRUE;
  gunsval[288][0][3] = TRUE;
  gunsval[288][0][4] = TRUE;
  gunsval[288][1][1] = TRUE;
  gunsval[289][0][0] = TRUE;
  gunsval[289][0][1] = TRUE;
  gunsval[289][0][3] = TRUE;
  gunsval[289][0][4] = TRUE;
  gunsval[289][1][1] = TRUE;
  gunsval[296][1][0] = TRUE;
  gunsval[296][3][1] = TRUE;
  gunsval[297][2][2] = TRUE;
  gunsval[297][3][1] = TRUE;
  gunsval[297][4][0] = TRUE;
  gunsval[297][4][1] = TRUE;
  gunsval[297][5][1] = TRUE;
  gunsval[298][3][0] = TRUE;
  gunsval[298][3][1] = TRUE;
  gunsval[298][4][0] = TRUE;
  gunsval[298][5][1] = TRUE;
  gunsval[299][2][1] = TRUE;
  gunsval[300][1][2] = TRUE;
  gunsval[300][2][1] = TRUE;
  gunsval[300][4][1] = TRUE;
  gunsval[304][5][0] = TRUE;
  gunsval[304][5][1] = TRUE;
  gunsval[304][5][3] = TRUE;
  gunsval[304][7][0] = TRUE;
  gunsval[304][7][1] = TRUE;
  gunsval[304][7][2] = TRUE;
  gunsval[304][7][3] = TRUE;
  gunsval[304][7][5] = TRUE;
  gunsval[304][7][6] = TRUE;
  gunsval[306][6][0] = TRUE;
  gunsval[306][7][0] = TRUE;
  gunsval[306][8][0] = TRUE;
  gunsval[306][8][1] = TRUE;
  gunsval[306][8][2] = TRUE;
  gunsval[306][8][4] = TRUE;
  gunsval[306][8][5] = TRUE;
  gunsval[306][8][6] = TRUE;
  gunsval[315][5][0] = TRUE;
  gunsval[315][5][1] = TRUE;
  gunsval[315][5][2] = TRUE;
  gunsval[315][5][4] = TRUE;
  gunsval[316][4][0] = TRUE;
  gunsval[316][4][1] = TRUE;
  gunsval[321][5][0] = TRUE;
  gunsval[321][5][1] = TRUE;
  gunsval[321][5][3] = TRUE;
  gunsval[321][8][1] = TRUE;
  gunsval[321][8][2] = TRUE;
  gunsval[322][0][0] = TRUE;
  gunsval[328][1][0] = TRUE;
  gunsval[328][1][1] = TRUE;
  gunsval[328][2][1] = TRUE;
  gunsval[328][2][2] = TRUE;
  gunsval[329][0][0] = TRUE;
  gunsval[329][0][1] = TRUE;
  gunsval[343][0][1] = TRUE;
  gunsval[343][5][0] = TRUE;
  gunsval[343][5][1] = TRUE;
  gunsval[343][5][3] = TRUE;
  gunsval[343][5][4] = TRUE;
  gunsval[343][5][5] = TRUE;
  gunsval[345][3][0] = TRUE;
  gunsval[345][3][1] = TRUE;
  gunsval[345][3][3] = TRUE;
  gunsval[345][3][4] = TRUE;
  gunsval[346][2][0] = TRUE;
  gunsval[346][2][1] = TRUE;
  gunsval[346][2][3] = TRUE;
  gunsval[347][8][0] = TRUE;
  gunsval[347][8][1] = TRUE;
  gunsval[347][8][2] = TRUE;
  gunsval[347][8][3] = TRUE;
  gunsval[347][8][4] = TRUE;
  gunsval[347][8][6] = TRUE;
  gunsval[347][8][7] = TRUE;
  gunsval[347][8][8] = TRUE;
  gunsval[348][0][0] = TRUE;
  gunsval[348][0][1] = TRUE;
  gunsval[348][0][2] = TRUE;
  gunsval[348][0][3] = TRUE;
  gunsval[348][0][5] = TRUE;
  gunsval[348][1][1] = TRUE;
  gunsval[348][2][1] = TRUE;
  gunsval[348][3][0] = TRUE;
  gunsval[348][3][1] = TRUE;
  gunsval[348][3][2] = TRUE;
  gunsval[349][5][1] = TRUE;
  gunsval[349][5][2] = TRUE;
  gunsval[349][5][3] = TRUE;
  gunsval[349][5][4] = TRUE;
  gunsval[349][5][5] = TRUE;
  gunsval[349][5][6] = TRUE;
  gunsval[349][5][7] = TRUE;
  gunsval[352][0][0] = TRUE;
  gunsval[352][0][1] = TRUE;
  gunsval[352][0][2] = TRUE;
  gunsval[352][0][3] = TRUE;
  gunsval[353][2][0] = TRUE;
  gunsval[353][2][2] = TRUE;
  gunsval[353][4][0] = TRUE;
  gunsval[353][4][1] = TRUE;
  gunsval[353][5][0] = TRUE;
  gunsval[353][5][2] = TRUE;
  gunsval[353][5][3] = TRUE;
  gunsval[353][5][4] = TRUE;
  gunsval[361][2][1] = TRUE;
  gunsval[361][3][0] = TRUE;
  gunsval[361][3][1] = TRUE;
  gunsval[361][3][2] = TRUE;
  gunsval[361][3][4] = TRUE;
  gunsval[365][1][1] = TRUE;
  gunsval[365][1][2] = TRUE;
  gunsval[365][3][0] = TRUE;
  gunsval[365][3][1] = TRUE;
  gunsval[365][3][3] = TRUE;
  gunsval[365][3][4] = TRUE;
  gunsval[365][4][1] = TRUE;
  gunsval[365][4][2] = TRUE;
  gunsval[366][0][1] = TRUE;
  gunsval[367][4][0] = TRUE;
  gunsval[367][4][1] = TRUE;
  gunsval[367][4][2] = TRUE;
  gunsval[367][4][4] = TRUE;
  gunsval[367][4][5] = TRUE;
  gunsval[367][4][6] = TRUE;
  gunsval[368][3][0] = TRUE;
  gunsval[368][3][1] = TRUE;
  gunsval[368][3][3] = TRUE;
  gunsval[368][3][4] = TRUE;
  gunsval[37][0][0] = TRUE;
  gunsval[37][1][1] = TRUE;
  gunsval[382][0][0] = TRUE;
  gunsval[383][1][0] = TRUE;
  gunsval[383][1][1] = TRUE;
  gunsval[383][1][2] = TRUE;
  gunsval[383][1][3] = TRUE;
  gunsval[383][1][5] = TRUE;
  gunsval[383][1][6] = TRUE;
  gunsval[383][1][7] = TRUE;
  gunsval[383][1][8] = TRUE;
  gunsval[384][0][1] = TRUE;
  gunsval[384][0][2] = TRUE;
  gunsval[384][2][0] = TRUE;
  gunsval[384][2][1] = TRUE;
  gunsval[384][2][2] = TRUE;
  gunsval[384][2][3] = TRUE;
  gunsval[384][2][5] = TRUE;
  gunsval[384][2][6] = TRUE;
  gunsval[384][2][7] = TRUE;
  gunsval[384][2][8] = TRUE;
  gunsval[384][5][0] = TRUE;
  gunsval[384][6][1] = TRUE;
  gunsval[38][0][0] = TRUE;
  gunsval[38][0][1] = TRUE;
  gunsval[38][0][2] = TRUE;
  gunsval[38][0][4] = TRUE;
  gunsval[38][1][1] = TRUE;
  gunsval[392][0][0] = TRUE;
  gunsval[392][0][1] = TRUE;
  gunsval[392][1][0] = TRUE;
  gunsval[392][1][1] = TRUE;
  gunsval[393][0][0] = TRUE;
  gunsval[393][0][1] = TRUE;
  gunsval[393][0][3] = TRUE;
  gunsval[393][0][4] = TRUE;
  gunsval[393][2][0] = TRUE;
  gunsval[393][4][0] = TRUE;
  gunsval[393][4][2] = TRUE;
  gunsval[393][4][3] = TRUE;
  gunsval[394][0][0] = TRUE;
  gunsval[394][1][0] = TRUE;
  gunsval[394][1][1] = TRUE;
  gunsval[394][1][2] = TRUE;
  gunsval[394][2][0] = TRUE;
  gunsval[394][2][2] = TRUE;
  gunsval[394][2][3] = TRUE;
  gunsval[394][2][4] = TRUE;
  gunsval[395][1][0] = TRUE;
  gunsval[395][2][1] = TRUE;
  gunsval[395][2][2] = TRUE;
  gunsval[395][2][3] = TRUE;
  gunsval[395][4][2] = TRUE;
  gunsval[395][5][1] = TRUE;
  gunsval[395][6][1] = TRUE;
  gunsval[397][0][0] = TRUE;
  gunsval[397][2][0] = TRUE;
  gunsval[397][2][1] = TRUE;
  gunsval[397][2][3] = TRUE;
  gunsval[397][2][4] = TRUE;
  gunsval[398][0][1] = TRUE;
  gunsval[398][3][0] = TRUE;
  gunsval[398][3][2] = TRUE;
  gunsval[398][3][3] = TRUE;
  gunsval[398][3][4] = TRUE;
  gunsval[398][4][1] = TRUE;
  gunsval[398][5][1] = TRUE;
  gunsval[399][3][0] = TRUE;
  gunsval[399][3][1] = TRUE;
  gunsval[399][3][2] = TRUE;
  gunsval[399][3][4] = TRUE;
  gunsval[3][6][0] = TRUE;
  gunsval[3][6][1] = TRUE;
  gunsval[3][6][2] = TRUE;
  gunsval[3][6][3] = TRUE;
  gunsval[3][6][4] = TRUE;
  gunsval[3][6][5] = TRUE;
  gunsval[40][0][0] = TRUE;
  gunsval[40][0][1] = TRUE;
  gunsval[40][0][2] = TRUE;
  gunsval[40][0][4] = TRUE;
  gunsval[40][1][0] = TRUE;
  gunsval[40][1][2] = TRUE;
  gunsval[41][0][0] = TRUE;
  gunsval[41][1][1] = TRUE;
  gunsval[41][2][1] = TRUE;
  gunsval[41][2][2] = TRUE;
  gunsval[42][0][0] = TRUE;
  gunsval[42][0][1] = TRUE;
  gunsval[42][0][2] = TRUE;
  gunsval[42][0][4] = TRUE;
  gunsval[42][1][0] = TRUE;
  gunsval[42][1][1] = TRUE;
  gunsval[43][0][0] = TRUE;
  gunsval[43][1][1] = TRUE;
  gunsval[43][1][2] = TRUE;
  gunsval[44][0][0] = TRUE;
  gunsval[44][0][1] = TRUE;
  gunsval[44][0][2] = TRUE;
  gunsval[44][0][4] = TRUE;
  gunsval[46][1][1] = TRUE;
  gunsval[46][1][2] = TRUE;
  gunsval[46][1][3] = TRUE;
  gunsval[46][2][0] = TRUE;
  gunsval[46][2][1] = TRUE;
  gunsval[46][2][3] = TRUE;
  gunsval[48][1][0] = TRUE;
  gunsval[48][1][1] = TRUE;
  gunsval[48][1][3] = TRUE;
  gunsval[48][2][0] = TRUE;
  gunsval[48][2][1] = TRUE;
  gunsval[48][2][3] = TRUE;
  gunsval[4][6][0] = TRUE;
  gunsval[4][6][1] = TRUE;
  gunsval[4][6][2] = TRUE;
  gunsval[4][6][3] = TRUE;
  gunsval[4][6][4] = TRUE;
  gunsval[4][6][5] = TRUE;
  gunsval[54][0][0] = TRUE;
  gunsval[54][0][2] = TRUE;
  gunsval[54][0][3] = TRUE;
  gunsval[54][0][4] = TRUE;
  gunsval[54][1][1] = TRUE;
  gunsval[54][1][2] = TRUE;
  gunsval[54][1][3] = TRUE;
  gunsval[59][1][2] = TRUE;
  gunsval[59][1][3] = TRUE;
  gunsval[59][4][0] = TRUE;
  gunsval[59][4][2] = TRUE;
  gunsval[59][4][3] = TRUE;
  gunsval[59][4][4] = TRUE;
  gunsval[59][4][5] = TRUE;
  gunsval[60][2][3] = TRUE;
  gunsval[60][2][4] = TRUE;
  gunsval[60][4][1] = TRUE;
  gunsval[60][4][2] = TRUE;
  gunsval[60][4][3] = TRUE;
  gunsval[60][4][4] = TRUE;
  gunsval[60][4][5] = TRUE;
  gunsval[63][3][0] = TRUE;
  gunsval[63][3][1] = TRUE;
  gunsval[63][3][3] = TRUE;
  gunsval[63][3][4] = TRUE;
  gunsval[71][1][1] = TRUE;
  gunsval[71][1][2] = TRUE;
  gunsval[71][1][3] = TRUE;
  gunsval[71][2][0] = TRUE;
  gunsval[71][2][1] = TRUE;
  gunsval[71][2][3] = TRUE;
  gunsval[74][0][1] = TRUE;
  gunsval[74][0][2] = TRUE;
  gunsval[74][0][3] = TRUE;
  gunsval[74][2][0] = TRUE;
  gunsval[74][2][1] = TRUE;
  gunsval[74][2][3] = TRUE;
  gunsval[82][0][2] = TRUE;
  gunsval[84][0][3] = TRUE;
  gunsval[85][0][3] = TRUE;
  gunsval[86][0][3] = TRUE;
  gunsval[87][0][3] = TRUE;
  gunsval[88][1][2] = TRUE;
  gunsval[89][2][2] = TRUE;
  gunsval[90][1][0] = TRUE;
  gunsval[92][1][2] = TRUE;
  gunsval[93][5][0] = TRUE;
  gunsval[93][5][1] = TRUE;
  gunsval[93][5][2] = TRUE;
  gunsval[93][5][4] = TRUE;
  gunsval[94][5][0] = TRUE;
  gunsval[94][5][1] = TRUE;
  gunsval[94][5][2] = TRUE;
  gunsval[94][5][4] = TRUE;
  gunsval[94][5][5] = TRUE;
  gunsval[94][5][6] = TRUE;
  gunsval[95][1][0] = TRUE;
  gunsval[95][1][1] = TRUE;
  gunsval[95][1][3] = TRUE;
  gunsval[95][2][1] = TRUE;
  gunsval[96][0][0] = TRUE;
  gunsval[96][0][2] = TRUE;
  gunsval[96][0][3] = TRUE;
  gunsval[96][1][0] = TRUE;
  gunsval[96][1][10] = TRUE;
  gunsval[96][1][2] = TRUE;
  gunsval[96][1][3] = TRUE;
  gunsval[96][1][4] = TRUE;
  gunsval[96][1][5] = TRUE;
  gunsval[96][1][6] = TRUE;
  gunsval[96][1][7] = TRUE;
  gunsval[96][1][8] = TRUE;
  gunsval[96][1][9] = TRUE;
  gunsval[96][2][0] = TRUE;
  gunsval[96][2][2] = TRUE;
  gunsval[96][3][0] = TRUE;
  gunsval[96][3][1] = TRUE;
  gunsval[96][3][10] = TRUE;
  gunsval[96][3][11] = TRUE;
  gunsval[96][3][2] = TRUE;
  gunsval[96][3][3] = TRUE;
  gunsval[96][3][5] = TRUE;
  gunsval[96][3][6] = TRUE;
  gunsval[96][3][7] = TRUE;
  gunsval[96][3][8] = TRUE;
  gunsval[96][3][9] = TRUE;
  gunsval[96][4][1] = TRUE;
  gunsval[96][4][2] = TRUE;
  gunsval[96][5][1] = TRUE;
  gunsval[97][0][1] = TRUE;
  gunsval[97][1][0] = TRUE;
  gunsval[97][1][1] = TRUE;
  gunsval[97][1][2] = TRUE;
  gunsval[97][2][0] = TRUE;
  gunsval[97][3][0] = TRUE;
  gunsval[97][3][1] = TRUE;
  gunsval[97][4][0] = TRUE;
  gunsval[97][4][2] = TRUE;
  gunsval[97][5][0] = TRUE;
  gunsval[97][5][2] = TRUE;
  gunsval[97][5][3] = TRUE;
  gunsval[98][2][0] = TRUE;
  gunsval[98][2][1] = TRUE;
  gunsval[98][3][1] = TRUE;
  gunsval[98][3][2] = TRUE;
  }

}

